
#+ echo=False
import os
import numpy

from biobakery_workflows import utilities, visualizations, files

from anadama2 import PweaveDocument

document=PweaveDocument()  

# get the variables for this document generation task
vars = document.get_vars()

# determine the document format
pdf_format = True if vars["format"] == "pdf" else False

# read in the DNA samples
dna_columns, dna_samples, dna_data = visualizations.qc_read_counts(document, vars["dna_read_counts"])

#' # Quality Control

#' <% visualizations.ShotGun.print_qc_intro_caption(len(dna_samples), dna_columns[2:]) %>

#' ## DNA Samples Quality Control

#' ### DNA Samples Tables of Filtered Reads

#+ echo=False

# create a table of the paired counts
document.write_table(["# Sample"]+dna_columns, dna_samples, dna_data,
    files.ShotGunVis.path("qc_counts",document.data_folder))

table_message=visualizations.show_table_max_rows(document, dna_data, dna_samples,
    dna_columns, "DNA reads", files.ShotGunVis.path("qc_counts"), format_data_comma=True)
        
#' <%= table_message %>

#+ echo=False
# compute and plot the microbial reads ratios
# compute ratios for each database used for qc
dna_microbial_reads, dna_microbial_labels = utilities.microbial_read_proportion_multiple_databases(dna_data, dna_columns)

# create a table of the microbial reads
document.write_table(["# Sample"]+dna_microbial_labels, dna_samples, 
    dna_microbial_reads, files.ShotGunVis.path("microbial_counts",document.data_folder))

table_message=visualizations.show_table_max_rows(document, dna_microbial_reads, dna_samples,
    dna_microbial_labels, "DNA microbial read proportion",
    files.ShotGunVis.path("microbial_counts"))  
    
#' <%= visualizations.ShotGun.captions["microbial_ratios"] %>  
        
#' <%= table_message %>
        
#' ### DNA Samples Plots of Filtered Reads

#+ echo=False
document.plot_grouped_barchart(numpy.transpose(dna_data), row_labels=dna_columns, 
    column_labels=dna_samples, title="DNA reads", ylabel="Read count (in millions)",
    legend_title="Filter", yaxis_in_millions=True)








#' % <% from anadama2 import PweaveDocument; document=PweaveDocument(); vars = document.get_vars(); print(vars["title"]) %>
#' % Project: <% print(vars["project"]) %>
#' % Date: <% import time; print(time.strftime("%m/%d/%Y")) %>
#+ echo=False

# get the variable settings from the data processing workflow
from anadama2.reporters import LoggerReporter
from biobakery_workflows import visualizations

try:
    workflow_settings = LoggerReporter.read_log(vars["log"],"variables")
except AttributeError:
    workflow_settings = []

# print a warning if the variables could not be read
if isinstance(workflow_settings, list):
    print("WARNING: Unable to read workflow settings from log file.")
    workflow_settings={}

maxee = workflow_settings.get("maxee","UNK")
trunc_len_max = workflow_settings.get("trunc_len_max","UNK")
percent_identity = workflow_settings.get("percent_identity","UNK")
min_cluster_size = workflow_settings.get("min_size","UNK")
dada_db = workflow_settings.get("dada_db", "UNK").upper()

method=vars["method"]

if method == "dada2" or method == "its":
    if method == "its":
        dada_db = "UNITE"
    dadadb_info="\nThe " + str(dada_db) + " database was used for taxonomy prediction."
    columns, samples, data = document.read_table(vars["counts_each_step"])

else:
    columns, samples, data = document.read_table(vars["read_count_table"])
    
    usearchintro="The " + str(len(samples)) + "  samples from this project were run through the standard 16S workflow.  \
        follows the UPARSE OTU analysis pipeline for OTU calling and taxonomy prediction with percent identity " \
        + str(percent_identity) + " and minimum cluster size of " + str(min_cluster_size) + "." \
        + "\n\nThe GreenGenes 16S RNA Gene Database version 13_8 was used for taxonomy prediction.\
        \n\nReads were filtered for quality control using a MAXEE score of " + str(maxee) + ". Filtered reads were \
        used to generate the OTUs. Reads not passing quality control were kept and used in the step \
        assigning reads to OTUs. First these reads were truncated to a max length of " + str(trunc_len_max) + " bases.\n"
        

import os
import numpy

# determine the document format
pdf_format=True if vars["format"] == "pdf" else False

#' <% if pdf_format: print("\clearpage") %>

#' # Introduction
#+ echo=False
#' <% if method== "its": print(visualizations.Sixteen_S.captions["itsintro"]) %>
#' <% if method == "dada2": print(visualizations.Sixteen_S.captions["dada2intro"]) %>
#' <% if method == "dada2" or method == "its": print(visualizations.Sixteen_S.captions["dada2stepsinfo"]) %>
#' <% if method == "dada2" or method == "its": print(dadadb_info) %>
#' <% if method != "dada2" and method != "its": print(usearchintro) %>

#+ echo=False


min_abundance=0.01
min_samples=10

from biobakery_workflows import utilities

# read in the read count table
# columns expected are total reads, reads that map to OTUs with taxonomy,
# and reads that map to OTUs without taxonomy

#' <% if pdf_format: print("\clearpage") %>


#' # Quality Control


#' <% if method == "dada2" or method == "its": print("## Forward Reads") %>   \
#' <% if method == "dada2" or method == "its": print("![FWD Read](" + vars["readF_qc"] + ")") %>
#' <% if method == "dada2" or method == "its": print("\clearpage")  %>

#' <% if method == "dada2" or method == "its": print("## Reverse Reads") %>   \
#' <% if method == "dada2" or method == "its": print("![REV Read](" + vars["readR_qc"] + ")") %>
#' <% if method == "dada2" or method == "its": print("\clearpage") %>
#+ echo=False

if method != "dada2" and method != "its":
    eestats_rows, eestats_columns, eestats_data, overall_stats = utilities.read_eestats2(vars["eestats_table"])
    document.show_table(eestats_data, eestats_rows, eestats_columns,"Expected error filter by read length",font="10")
    
#' <% if method != "dada2" and method != "its": print("The general stats for this data set are:" + str(overall_stats)) %>
#' <% if method != "dada2" and method != "its": print("This table shows the number of reads based on length for different error filters.") %>

#' <% if method == "dada2" or method == "its": print("# Error rates") %>
    
#' <% if method == "dada2" or method == "its": print(visualizations.Sixteen_S.captions["dada2errorintro"]) %>
    
#' <% if method == "dada2" or method == "its": print("## Forward Reads") %>  \
#' <% if method == "dada2" or method == "its": print("![FWD Error Rates](" + vars["error_ratesF"] +")") %>
#' <% if method == "dada2" or method == "its": print("\clearpage") %>

#' <% if method == "dada2" or method == "its": print("## Reverse Reads") %>   \
#' <% if method == "dada2" or method == "its": print("![REV Error Rates](" + vars["error_ratesR"] + ")") %>

#' <% if method == "dada2" or method == "its": print(visualizations.Sixteen_S.captions["dada2errorinfo"]) %>
#' <% if pdf_format: print("\clearpage") %>
      
#+ echo=False
# if picard files are provided, then plot those that do not meet a threshold
picard_text = ""
if vars["picard"]:
    picard_min_threshold=20
    # read through all picard files, capturing those based on the threshold
    data_below_threshold={}
    samples_above_threshold=[]
    for file in utilities.get_files(vars["picard"], extension=vars["picard_ext"]):
        picard_sample_name = os.path.basename(file).replace("."+vars["picard_ext"],"")
        picard_data, below_threshold = utilities.read_picard(file, threshold=picard_min_threshold)
        if picard_data:
            if below_threshold:
                data_below_threshold[picard_sample_name]=[a[1] for a in picard_data]
            else:
                samples_above_threshold.append(picard_sample_name)

    # plot each sample that is below the threshold
    for sample, picard_data in data_below_threshold.items():
        document.plot_barchart(picard_data,
            title="Picard quality scores for "+sample, xlabel="Cycle", ylabel="Quality Score")
    
    
    if len(data_below_threshold.keys()) > 0:
        picard_text="Only the samples with at least one quality score below the threshold "+\
            " ( "+str(picard_min_threshold)+" ) are shown as quality scores for each base. These samples are: "+\
            ",".join(list(data_below_threshold.keys()))+"."
    else:
        picard_text="All samples had all quality scores above the threshold "+\
            " ( "+str(picard_min_threshold)+" )."
    
    # list those that were above the threshold
    above_threshold_list=",".join(samples_above_threshold)
    if above_threshold_list:
        picard_text+=" The following samples did not have any quality scores below the threshold: " + above_threshold_list + "."
    else:
        picard_text+=" None of the samples had all quality scores above the threshold."


#' <% if picard_text: print(picard_text) %>
#' # Read Count
#+ echo=False

# sort the samples/data by read count with the largest original read count first
total_reads=[row[0] for row in data]
sorted_samples, sorted_total_reads = utilities.sort_data(total_reads, samples)
sorted_all_read_data = [data[samples.index(sample)] for sample in sorted_samples]

if method == "dada2" or method == "its":

    filtered_reads = [row[1] for row in sorted_all_read_data]
    merged_reads = [row[3] for row in sorted_all_read_data]
    tabled_reads = [row[4] for row in sorted_all_read_data]
    nochim_reads = [row[5] for row in sorted_all_read_data]
    # plot the read counts
    document.plot_grouped_barchart([sorted_total_reads,filtered_reads,merged_reads,tabled_reads,nochim_reads],
           ["Original","Filtered","Merged","Tabled","Nochimera"], sorted_samples,
           title="Read counts by Sample", xlabel="Samples", ylabel="Total Reads")
    # plot grouped taxonomy for all categorical data provided
    if visualizations.metadata_provided(vars):
        categorical_metadata, ordered_sorted_data, ordered_metadata, samples_found = visualizations.merge_categorical_metadata(vars, sorted_samples,
            [total_reads,filtered_reads,merged_reads,tabled_reads,nochim_reads])
        for cat_metadata in categorical_metadata:
            visualizations.plot_grouped_taxonomy_subsets(document, ordered_sorted_data, cat_metadata, 
                ["total","filtered","merged","tabled","nochimera"], samples_found, 
                title="Read counts by sample", ylabel="Total Reads", legend_title="")

else:
    known_reads = [row[1] for row in sorted_all_read_data]
    unknown_reads = [row[2] for row in sorted_all_read_data]
    unmapped_reads = [row[0]-(row[1]+row[2]) for row in sorted_all_read_data]
    # plot the read counts
    document.plot_stacked_barchart([known_reads,unknown_reads,unmapped_reads], ["classified","unclassified","unmapped"], sorted_samples, 
        title="Read counts by Sample", ylabel="Total Reads", xlabel="Samples")
    # plot grouped taxonomy for all categorical data provided
    if visualizations.metadata_provided(vars):
        categorical_metadata, ordered_sorted_data, ordered_metadata, samples_found = visualizations.merge_categorical_metadata(vars, sorted_samples,
            [known_reads,unknown_reads,unmapped_reads])
        for cat_metadata in categorical_metadata:
            visualizations.plot_grouped_taxonomy_subsets(document, ordered_sorted_data, cat_metadata, ["classified","unclassified","unmapped"],
                samples_found, title="Read counts by sample", ylabel="Total Reads", legend_title="")

#' <% if method == "dada2" or method == "its": print(visualizations.Sixteen_S.captions["dada2countsinfo"]) %>
#' <% if method != "dada2" and method != "its": print(visualizations.Sixteen_S.captions["usearchcountsinfo"]) %>

#' <% if pdf_format: print("\clearpage") %>

#' # Taxonomy
    
#' <% if method == "dada2" or method == "its": print(visualizations.Sixteen_S.captions["dada2taxinfo"]) %>
#' <% if method == "dada2" or method == "its": print(dadadb_info) %>

#' ## Genera
 
#+ echo=False

# read in the otu table data
samples, ids, taxonomy, data = utilities.read_otu_table(vars["otu_table"])

# plot the top taxa by genus level, plotting the relative abundance values
max_taxa=15

# get the relative abundance values for the samples
relab_data = utilities.relative_abundance(data, percent=True)

# get the top taxa by genus level
max_taxa = 15
sorted_samples, sorted_top_data, top_data, top_taxa_short_names, legend_size = visualizations.get_top_taxonomy_by_level(taxonomy, samples, relab_data, max_taxa)

# add other to the taxonomy data, other represents total genera not shown on plot
top_taxa_short_names_plus_other, sorted_top_data_plus_other = visualizations.fill_taxonomy_other(top_taxa_short_names, sorted_top_data)

document.plot_stacked_barchart(sorted_top_data_plus_other, row_labels=top_taxa_short_names_plus_other,
    column_labels=sorted_samples, title="Top "+str(max_taxa)+" genera by average abundance",
    ylabel="Relative abundance", legend_title="Genera", legend_style="italic", legend_size=legend_size)

#+ echo=False
# plot grouped and average barplots for metadata if provided
visualizations.plot_grouped_and_average_barplots_taxonomy(document, vars, sorted_samples, sorted_top_data_plus_other, top_taxa_short_names_plus_other, max_taxa, feature="genera")

#' <% if pdf_format: print("\clearpage") %>

#' ## Terminal Taxa

#+ echo=False

# plot the relative abundance of the top terminal taxa
# get the terminal taxa
terminal_taxa_relab, terminal_data_relab = utilities.terminal_taxa(taxonomy, relab_data)
# get the top rows of terminal taxa
top_terminal_taxa, top_terminal_data = utilities.top_rows(terminal_taxa_relab, terminal_data_relab, max_taxa, function="average")

# reduce the taxa names to just the most specific identifier
shorted_names=utilities.taxonomy_trim(top_terminal_taxa)

# sort the data with the samples with the top terminal taxa first
sorted_samples_terminal, sorted_data_terminal = utilities.sort_data(top_terminal_data[0], samples)
transpose_top_terminal_data = numpy.transpose(top_terminal_data)
sorted_top_terminal_data = numpy.transpose([transpose_top_terminal_data[samples.index(sample)] for sample in sorted_samples_terminal])

# add the remaining terminal taxa as "other" to the data
shorted_names_plus_other, sorted_top_terminal_data_plus_other = visualizations.fill_taxonomy_other(shorted_names, sorted_top_terminal_data)

document.plot_stacked_barchart(sorted_top_terminal_data_plus_other, row_labels=shorted_names_plus_other,
    column_labels=sorted_samples_terminal, title="Top "+str(max_taxa)+" terminal taxa by average abundance",
    ylabel="Relative abundance", legend_title="Terminal taxa")

visualizations.plot_grouped_and_average_barplots_taxonomy(document, vars, sorted_samples_terminal, sorted_top_terminal_data_plus_other, shorted_names_plus_other, max_taxa, feature="terminal taxa")

#' # Heatmaps

#+ echo=False
max_sets_heatmap=25

#' <%= visualizations.ShotGun.format_caption("heatmap_intro",max_sets=max_sets_heatmap,type="genera and terminal taxa",method="Spearman and Bray-Curtis") %>

#' ## Genera

#+ echo=False
# update the figure size based on output format for the heatmaps
utilities.change_pweave_figure_size_heatmap(pdf_format)

visualizations.plot_heatmap(document,vars,samples,top_taxa_short_names,top_data,
    pdf_format,"Top {} genera by average abundance (Spearman)".format(max_sets_heatmap),max_sets_heatmap)
utilities.reset_pweave_figure_size()
#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,top_taxa_short_names,top_data,
    pdf_format,"Top {} genera by average abundance (Bray-Curtis)".format(max_sets_heatmap),max_sets_heatmap,method="lbraycurtis")
utilities.reset_pweave_figure_size()



#' ## Terminal Taxa

#+ echo=False
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,shorted_names,top_terminal_data,
    pdf_format,"Top {} terminal taxa by average abundance (Spearman)".format(max_sets_heatmap),max_sets_heatmap)
utilities.reset_pweave_figure_size()

#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,shorted_names,top_terminal_data,
    pdf_format,"Top {} terminal taxa by average abundance (Bray-Curtis)".format(max_sets_heatmap),max_sets_heatmap,method="lbraycurtis")
utilities.reset_pweave_figure_size()


#' # Ordination

#' ## Genera

#+ echo=False
import numpy

# filter then get top genera
filtered_taxonomy_all, filtered_relab_data_all = utilities.filter_taxa_abundance(taxonomy, relab_data, min_abundance, min_samples)
samples_genera, sorted_top_data_genera, top_data_genera, top_taxa_genera, legend_size = visualizations.get_top_taxonomy_by_level(filtered_taxonomy_all, samples, filtered_relab_data_all, max_sets_heatmap)

# provide data as values [0-1] organized as samples as columns and features as rows
top_filtered_data_pcoa=numpy.array(sorted_top_data_genera)/100.0
document.show_pcoa(samples_genera, top_taxa_genera, top_filtered_data_pcoa, title="PCoA Ordination of top {} genera using Bray-Curtis similarity".format(max_sets_heatmap))

#' For the PCoA plots, relative abundances are passed through a basic filter requiring each taxon
#' have at least <% print(min_abundance)%> % abundance in at least
#' <% print(min_samples) %> % of all samples.

#+ echo=False
visualizations.show_pcoa_metadata(document, vars, samples_genera, top_taxa_genera, top_filtered_data_pcoa,
    title="PCoA Ordination of top {} genera".format(max_sets_heatmap))

#' ## Terminal taxa

#+ echo=False
# plot the top terminal node taxa in a PCoA

# filter the data by min abundance and min samples
filtered_taxonomy, filtered_data = utilities.filter_taxa_abundance(terminal_taxa_relab, terminal_data_relab, min_abundance, min_samples)

# get the top set of terminal taxa
top_filtered_taxonomy, top_filtered_data = utilities.top_rows(filtered_taxonomy, filtered_data, max_sets_heatmap, function="average")

# provide data as values [0-1] organized as samples as columns and features as rows
top_filtered_data_pcoa=numpy.array(top_filtered_data)/100.0
document.show_pcoa(samples, top_filtered_taxonomy, top_filtered_data_pcoa, title="PCoA Ordination of top {} terminal taxa using Bray-Curtis similarity".format(max_sets_heatmap))

visualizations.show_pcoa_metadata(document, vars, samples, top_filtered_taxonomy, top_filtered_data_pcoa,
    title="PCoA Ordination of top {} terminal taxa".format(max_sets_heatmap))

#' <% if pdf_format: print("\clearpage") %>



#!/usr/bin/env python

"""
bioBakery Workflows: whole metagenome and metatranscriptome shotgun workflow

Copyright (c) 2016 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os

# import the workflow class from anadama2
from anadama2 import Workflow

# import the library of biobakery_workflow tasks for shotgun sequences
from biobakery_workflows.tasks import shotgun

# import the utilities functions, config settings, and file names from biobakery_workflows
from biobakery_workflows import utilities, config, files

# create a workflow instance, providing the version number and description
# remove the input folder option as it will be replaced with two input folder options
workflow = Workflow(version="0.1", remove_options=["input"],
                    description="A workflow for whole metagenome and metatranscriptome shotgun sequences")

# add the custom arguments to the workflow
workflow_config = config.ShotGun()
workflow.add_argument("input-metagenome",desc="the input folder of whole metagenome shotgun sequences", required=True)
workflow.add_argument("input-metatranscriptome",desc="the input folder of whole metatranscriptome shotgun sequences", required=True)
workflow.add_argument("input-mapping",desc="the mapping file of metatranscriptome samples to metagenome samples")
workflow.add_argument("input-extension", desc="the input file extension", default="fastq.gz", choices=["fastq.gz","fastq","fq.gz","fq"])
workflow.add_argument("threads", desc="number of threads/cores for each task to use", default=1)
workflow.add_argument("pair-identifier", desc="the string to identify the first file in a pair", default=".R1")
workflow.add_argument("bypass-norm-ratio", desc="bypass the rna/dna normalization computation", action="store_true")
workflow.add_argument("qc-options", desc="additional options when running the QC step", default="")
workflow.add_argument("remove-intermediate-output", desc="remove intermediate output files", action="store_true")
workflow.add_argument("bypass-strain-profiling", desc="do not run the strain profiling tasks", action="store_true")

# get the arguments from the command line
args = workflow.parse_args()

# get all input files with the input extension provided on the command line
input_files_metagenome = utilities.find_files(args.input_metagenome, extension=args.input_extension, exit_if_not_found=True)
input_files_metatranscriptome = utilities.find_files(args.input_metatranscriptome, extension=args.input_extension, exit_if_not_found=True)

### STEP #1: Run quality control on all input files ###
wms_output_folder = os.path.join(args.output,files.ShotGun.wmgx_folder_name)
wts_output_folder = os.path.join(args.output,files.ShotGun.wmtx_folder_name)
wms_qc_output_files, wms_filtered_read_count = shotgun.quality_control(workflow, 
    input_files_metagenome, args.input_extension, wms_output_folder, args.threads, 
    [workflow_config.kneaddata_db_human_genome], 
    args.pair_identifier, args.qc_options, args.remove_intermediate_output)
wts_qc_output_files, wts_filtered_read_count = shotgun.quality_control(workflow, 
    input_files_metatranscriptome, args.input_extension, wts_output_folder, args.threads, 
    [workflow_config.kneaddata_db_human_genome,workflow_config.kneaddata_db_human_metatranscriptome,workflow_config.kneaddata_db_rrna], 
    args.pair_identifier, args.qc_options, args.remove_intermediate_output)

# if the original files were gzipped they will not be compressed after qc
args.input_extension = args.input_extension.replace(".gz","")

### STEP #2: Run taxonomic profiling on all of the metagenome filtered files (and metatranscriptome if mapping not provided)###
wms_taxonomic_profile, wms_taxonomy_tsv_files, wms_taxonomy_sam_files = shotgun.taxonomic_profile(workflow,
    wms_qc_output_files,wms_output_folder,args.threads,args.input_extension)

if not args.input_mapping:
    wts_taxonomic_profile, wts_taxonomy_tsv_files, wts_taxonomy_sam_files = shotgun.taxonomic_profile(workflow,
        wts_qc_output_files,wts_output_folder,args.threads,args.input_extension)   

### STEP #3: Run functional profiling on all of the filtered files ###

# run the wms samples with their taxonomy files
wms_genes_relab, wms_ecs_relab, wms_path_relab, wms_genes, wms_ecs, wms_path = shotgun.functional_profile(workflow,
    wms_qc_output_files,args.input_extension,wms_output_folder,args.threads,wms_taxonomy_tsv_files)

# provide the taxonomy files for the wms samples to the wts samples, if mapping file provided
if args.input_mapping:
    # get the mapped taxonomy files for the quality controlled input files
    filtered_fastq, matched_taxonomic_profile = utilities.match_files(wts_qc_output_files,wms_taxonomy_tsv_files,args.input_mapping)
    wts_genes_relab, wts_ecs_relab, wts_path_relab, wts_genes, wts_ecs, wts_path = shotgun.functional_profile(workflow,
        filtered_fastq,args.input_extension,wts_output_folder,args.threads,matched_taxonomic_profile,args.remove_intermediate_output)
else:
    # if no mapping file is provided then run to get a taxonomic profile from the wts samples
    wts_genes_relab, wts_ecs_relab, wts_path_relab, wts_genes, wts_ecs, wts_path = shotgun.functional_profile(workflow,
        wts_qc_output_files,args.input_extension,wts_output_folder,args.threads,wts_taxonomy_tsv_files,args.remove_intermediate_output)

### STEP #4: Compute the normalized functional abundances based on the rna/dna ratio
if not args.bypass_norm_ratio:
    norm_ratio_genes, norm_ratio_ecs, norm_ratio_pathway = shotgun.norm_ratio(workflow, 
        wms_genes, wms_ecs, wms_path, wts_genes, wts_ecs, wts_path, args.output, args.input_mapping)

### STEP #4: Run strain profiling
if not args.bypass_strain_profiling:
    shotgun.strain_profile(workflow,wms_taxonomy_sam_files,args.output,args.threads,
        workflow_config.strainphlan_db_reference,workflow_config.strainphlan_db_markers,wms_taxonomic_profile)

# start the workflow
workflow.go()



#!/usr/bin/env python

""" This script will generate dual barcode file from index(barcode) files.
    It is used for demultiplexing dual indexed reads.
"""

import argparse
import sys
import os, fnmatch
from biobakery_workflows.tasks import general


def parse_arguments(args):
    """ Parse the arguments from the user """

    parser = argparse.ArgumentParser(
        description="Generate dual barcode file for demultiplexing\n",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "--input",
        help="input folder with dual indexed reads and barcodes\n[REQUIRED]",
        required=True)
    parser.add_argument(
        "--barcode-pair-identifier", default="_1",
        help="pair identifier for barcodes(index) files\n[REQUIRED]",
        required=False)

    return parser.parse_args()

def main():
    # parse arguments from the user
    args = parse_arguments(sys.argv)
    pair_id1 = args.barcode_pair_identifier
    pair_id2 = pair_id1.replace("1","2")
    barcode_files1 = fnmatch.filter(os.listdir(args.input), "*barcode"+ pair_id1 + ".fastq*")
    barcode_files2 = fnmatch.filter(os.listdir(args.input), "*barcode" + pair_id2 + ".fastq*")
    barcode_files = barcode_files1 + barcode_files2
    barcode_files = [os.path.join(args.input, file) for file in barcode_files]
    # name dual barcode file
    dual_barcode_file = os.path.join(args.input,"dual_barcode_file.txt")
    # generate dual barcode file
    general.generate_dual_barcode(barcode_files, dual_barcode_file)

if __name__ == "__main__":
    main()


# This script will replace the sequence ids with sample ids in all data products
# in the "output_folder", copied from the input_folder using the mappings in the "mapping_file"
# To run: $ python rename_data_products.py infolder outfolder mapping_file.tsv

import sys
import os
import subprocess
import time

input_folder = sys.argv[1]
output_folder = sys.argv[2]
mapping_file = sys.argv[3]

command1 = "sed -i 's/^old/new/g' "
command2 = "sed -i 's/\told/\tnew/g' "

# copy the existing files to the new folder
cmmd=["cp","-r",input_folder,output_folder]
print(cmmd)
subprocess.check_output(cmmd)

# read in the mappings
file_handle = open(mapping_file)
header = file_handle.readline()
mapping={}
for line in file_handle:
    data = line.rstrip().split("\t")
    mapping[data[0]]=data[1]

# rename all sample names in all files in product folder
for file in os.listdir(output_folder):
    file = os.path.join(output_folder,file)
    for seq_id, sample_id in mapping.items():
        for command in [command1, command2]:
            new_command = command.replace("old",seq_id).replace("new",sample_id)+file
            print(new_command)
            subprocess.check_output(new_command,shell=True)
            time.sleep(1)




#!/usr/bin/env python

"""
bioBakery Workflows: isolate assembly

Copyright (c) 2016 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""
import sys
import os, fnmatch

# import the workflow class from anadama2
from anadama2 import Workflow
from anadama2.tracked import TrackedExecutable

# import the library of biobakery_workflow tasks for shotgun sequences
from biobakery_workflows.tasks import shotgun, general

# import the utilities functions and config settings from biobakery_workflows
from biobakery_workflows import utilities, config

# create a workflow instance, providing the version number and description
# the version number will appear when running this script with the "--version" option
# the description will appear when running this script with the "--help" option
workflow = Workflow(version="0.1", description="A workflow for isolate assembly")

# add the custom arguments to the workflow
workflow_config = config.ShotGun()
workflow.add_argument("species-name", desc="the species name", required=True)
workflow.add_argument("input-extension", desc="the input file extension", default="fastq.gz", choices=["fastq.gz","fastq","fq.gz","fq"])
workflow.add_argument("threads", desc="number of threads/cores for each task to use", default=1)
workflow.add_argument("pair-identifier", desc="the string to identify the first file in a pair", default="_R1_001")
workflow.add_argument("reference-database", desc="the path to the reference database for quality assessment", default="")
workflow.add_argument("dbcan-path", desc="the path to the run_dbcan.py script", default="/app/")

# get the arguments from the command line
args = workflow.parse_args()

# get all input files with the input extension provided on the command line
# return an error if no files are found
input_files = utilities.find_files(args.input, extension=args.input_extension, exit_if_not_found=True)

### STEP #1: Run quality control on all input files ###
sample_names=utilities.sample_names(input_files,args.input_extension)
input_pair1, input_pair2 = utilities.paired_files(input_files, args.input_extension, args.pair_identifier)
paired = False
if input_pair1:
    sample_names=utilities.sample_names(input_pair1,args.input_extension,args.pair_identifier)
    qc_targets=[utilities.name_files([name+".trimmed.1.fastq",name+".trimmed.2.fastq",name+".trimmed.single.1.fastq",name+".trimmed.single.2.fastq",name+".trimmed.single.12.fastq"], args.output, subfolder="kneaddata", create_folder=True) for name in sample_names]
    paired = True
    for target_set,input_R1,input_R2,name in zip(qc_targets,input_pair1,input_pair2,sample_names):
        workflow.add_task(
            "kneaddata --run-fastqc-start --input [depends[0]] --input [depends[1]] --output [args[0]] --threads [args[1]] --output-prefix [args[2]] && cat [args[3]] [args[4]] > [targets[2]]",
            depends=[input_R1, input_R2, TrackedExecutable("kneaddata")],
            targets=[target_set[0],target_set[1],target_set[4]],
            args=[os.path.dirname(target_set[0]),args.threads,name,target_set[2],target_set[3]])
else:
    qc_targets=utilities.name_files(sample_names, args.output, subfolder="kneaddata", create_folder=True, extension="trimmed.fastq")
    for target_file,input_file,name in zip(qc_targets,input_files,sample_names):
        workflow.add_task(
            "kneaddata --run-fastqc-start --input [depends[0]] --output [args[0]] --threads [args[1]] --output-prefix [args[2]]",
            depends=[input_file, TrackedExecutable("kneaddata")],
            targets=[target_file],
            args=[os.path.dirname(target_file),args.threads,name])

### STEP #2: Run assembly ###
assembly_inputs = ""
assembly_depends=[]
assembly_targets = utilities.name_files("contigs.fasta", args.output, subfolder="spades", create_folder=True)
if paired:
    for i, inputs in enumerate(qc_targets):
        assembly_inputs += "--pe{0}-1 {1} --pe{0}-2  {2} --pe{0}-s {3} ".format(i+1, inputs[0], inputs[1], inputs[4])
        assembly_depends+=[inputs[0],inputs[1],inputs[4]]
    workflow.add_task(
        "spades.py "+assembly_inputs+" --careful --cov-cutoff auto -o [args[0]] --threads [args[1]]",
        depends=assembly_depends+[TrackedExecutable("spades.py")],
        targets=assembly_targets,
        args=[os.path.dirname(assembly_targets),args.threads])
else:
    for input_file in qc_targets:
        assembly_inputs += "-s {0} ".format(input_file)
    workflow.add_task(
        "spades.py "+assembly_inputs+" --careful --cov-cutoff auto -o [args[0]] --threads [args[1]]",
        depends=qc_targets+[TrackedExecutable("spades.py")],
        targets=assembly_targets,
        args=[os.path.dirname(assembly_targets),args.threads])

### STEP #3: Annotate assembly ###
annotation_targets = utilities.name_files([args.species_name +".faa", args.species_name +".ffn"], args.output, subfolder="prokka")
workflow.add_task(
    "prokka --outdir [args[0]] --prefix [args[1]] [depends[0]] --cpus [args[2]]",
    depends=[assembly_targets,TrackedExecutable("prokka")],
    targets=annotation_targets,
    args=[os.path.dirname(annotation_targets[0]),args.species_name,args.threads])

### STEP #4: Run quality assessment with quast ###
quast_targets = utilities.name_files("quast_stdout.txt", args.output, subfolder="quast", create_folder=True)
optional_database = ""
if args.reference_database:
    optional_database = " -r " + args.reference_database
workflow.add_task(
    "quast [depends[0]] --output-dir [args[0]] --threads [args[1]] "+optional_database+" > [targets[0]]",
    depends=[assembly_targets,TrackedExecutable("quast")],
    targets=quast_targets,
    args=[os.path.dirname(quast_targets), args.threads])

### STEP #5: Run quality assessment with checkm ###
checkm_targets = utilities.name_files("checkm_stdout.log", args.output, subfolder="checkm", create_folder=True)
workflow.add_task(
    "checkm lineage_wf [args[0]] [args[1]] > [targets[0]]",
    depends=[annotation_targets[0],TrackedExecutable("checkm",version_command="{}")],
    targets=checkm_targets,
    args=[os.path.dirname(annotation_targets[0]), os.path.dirname(checkm_targets)])

### STEP #6: Functional annotations with emapper ###
functional_targets = utilities.name_files(["eggnog_mapper.log", "eggnog_mapper.emapper.annotations"], args.output, subfolder="eggnog_mapper", create_folder=True)
workflow.add_task(
    "emapper.py -o [args[0]] --output_dir [args[1]] -i [depends[0]] -m diamond --cpu [args[2]] > [targets[0]]",
    depends=[annotation_targets[0],TrackedExecutable("emapper.py")],
    targets=functional_targets,
    args=[os.path.basename(functional_targets[0]).split(".")[0],os.path.dirname(functional_targets[0]),args.threads])

### STEP #7: Functional annotations with dbcan ###
dbcan_targets = utilities.name_files("overview.txt", args.output, subfolder="run_dbcan")
workflow.add_task(
    "cd [args[0]] && python run_dbcan.py [depends[0]] protein --out_dir [args[1]]",
    depends=annotation_targets[0],
    targets=dbcan_targets,
    args=[args.dbcan_path,os.path.dirname(dbcan_targets)],
    name="run_dbcan")

### STEP #8: Add functional annotations to genome files ###
final_genomes = utilities.name_files([args.species_name +".faa", args.species_name +".ffn"], args.output, subfolder="annotated_genomes", create_folder=True)
for annotation_file, genome_file in zip(annotation_targets, final_genomes):
    workflow.add_task(
        "annotate_genome.py --input-genome [depends[0]] --input-dbcan [depends[1]] --input-eggnog [depends[2]] --output [targets[0]]",
        depends=[annotation_file,dbcan_targets,functional_targets[1]],
        targets=genome_file)

# start the workflow
workflow.go()




#+ echo=False
import os
import math

min_abundance=0.01
min_samples=10
max_sets_heatmap=25
max_sets_barplot=15

from biobakery_workflows import utilities, visualizations, files

from anadama2 import PweaveDocument

document=PweaveDocument()  

# get the variables for this document generation task
vars = document.get_vars()

# determine the document format
pdf_format = True if vars["format"] == "pdf" else False

#' # Taxonomic Profiling of Metagenomic Reads

#' This report section contains information about the taxonomy
#' for all DNA samples. These samples were
#' run through [MetaPhlAn2](http://huttenhower.sph.harvard.edu/metaphlan2).

#' Taxonomic abundances are passed through a basic filter requiring each species or genus
#' to have at least <% print(min_abundance)%> % abundance in at least 
#' <% print(min_samples) %> % of all samples.

#+ echo=False

# read in the taxonomy data
samples, taxonomy, data = document.read_table(vars["taxonomic_profile"])

# remove extra information from sample name if included from workflow join
samples=[s.replace("_taxonomic_profile","") for s in samples]

# filter to only include data for the species level
# get the rows with species but not strain information
species_taxonomy, species_data = utilities.filter_taxa_level_metaphlan2_format(taxonomy,data)

# now filter species also applying min abundance and min samples
filtered_species_taxonomy, filtered_species_data = utilities.filter_taxa_level_metaphlan2_format(taxonomy,
    data, min_abundance=min_abundance, min_samples=min_samples)

# filter to only include genus level
genera_taxonomy, genera_data = utilities.filter_taxa_level_metaphlan2_format(taxonomy,data,level=5)

# filter genus level plus min abundance and min samples
filtered_genera_taxonomy, filtered_genera_data = utilities.filter_taxa_level_metaphlan2_format(taxonomy,
    data, min_abundance=min_abundance, min_samples=min_samples, level=5)

#' A total of <% print(len(species_taxonomy)) %> species and <% print(len(genera_taxonomy)) %> genera were identified. 
#' After basic filtering <% print(len(filtered_species_taxonomy)) %> species and <% print(len(filtered_genera_taxonomy)) %> genera remained. 

#' ## Taxonomic Count Table

#+ echo=False
import numpy

# count the number of species that pass filters for each sample
def count_filtered_columns(data, min):
    # first transpose the data
    data=numpy.transpose(data)
    # get the items from a row that pass the min filter
    # then count the total items per row
    return [len(list(filter(None,filter(lambda x: x>min,row)))) for row in data]

species_counts=count_filtered_columns(species_data, min=0)
species_counts_after_filter=count_filtered_columns(filtered_species_data, min=0)
genera_counts=count_filtered_columns(genera_data, min=0)
genera_counts_after_filter=count_filtered_columns(filtered_genera_data, min=0)

all_taxa_counts=[[a,b,c,d] for a,b,c,d in zip(species_counts, species_counts_after_filter, genera_counts, genera_counts_after_filter)]

# create a table of the data in the output folder
taxa_counts_column_names = ["# Sample","Species","Species filtered","Genera","Genera filtered"]
document.write_table(taxa_counts_column_names, samples, all_taxa_counts,
    files.ShotGunVis.path("taxa_counts",document.data_folder))

# show the table, reducing the rows if there are lots of samples
table_message=visualizations.show_table_max_rows(document, all_taxa_counts, samples,
    taxa_counts_column_names[1:],"Total taxa per sample",files.ShotGunVis.path("taxa_counts"))

#' <%= table_message %>

#' ## Ordination

#' ### Species

#+ echo=False
# get the top species by average abundance
top_taxonomy, top_data = utilities.top_rows(species_taxonomy, species_data, max_sets_heatmap,
    function="average") 

# compute the pcoa and plot
# provide data as range of [0-1] organised as samples as rows and features as columns
pcoa_data=numpy.array(top_data)/100.0
caption=document.show_pcoa(samples,top_taxonomy,pcoa_data,"PCoA Ordination of top {} species using Bray-Curtis similarity".format(max_sets_heatmap))

#' <% print(caption) %>

#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
visualizations.show_pcoa_metadata(document, vars, samples, top_taxonomy, pcoa_data,
    title="PCoA Ordination of top {} species".format(max_sets_heatmap))

#' ### Genera

#+ echo=False
top_taxonomy_genera, top_data_genera = utilities.top_rows(genera_taxonomy, genera_data, max_sets_heatmap,
    function="average")

# compute and plot pcoa
pcoa_data_genera=numpy.array(top_data_genera)/100.0
caption_genera=document.show_pcoa(samples,top_taxonomy_genera,pcoa_data_genera,
    "PCoA Ordination of top {} genera using Bray-Curtis similarity".format(max_sets_heatmap),feature_types="genera")

#' <% print(caption_genera) %>

#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
visualizations.show_pcoa_metadata(document, vars, samples, top_taxonomy_genera, pcoa_data_genera,
    title="PCoA Ordination of top {} genera".format(max_sets_heatmap))

#' ## Heatmaps

#' <%= visualizations.ShotGun.format_caption("heatmap_intro",max_sets=max_sets_heatmap,type="species and genera",method="Spearman and Bray-Curtis") %>

#' ### Species

#+ echo=False
# update the figure size based on output format for the heatmaps
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,top_taxonomy,top_data,
    pdf_format, "Top {} species by average abundance (Spearman)".format(max_sets_heatmap),max_sets_heatmap)
utilities.reset_pweave_figure_size()
#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,top_taxonomy,top_data,
    pdf_format,"Top {} species by average abundance (Bray-Curtis)".format(max_sets_heatmap),max_sets_heatmap,method="lbraycurtis")
utilities.reset_pweave_figure_size()

#' ### Genera

#+ echo=False
# update the figure size based on output format for the heatmaps
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,top_taxonomy_genera,top_data_genera,
    pdf_format, "Top {} genera by average abundance (Spearman)".format(max_sets_heatmap),max_sets_heatmap)
utilities.reset_pweave_figure_size()
#' <% if pdf_format: print("\clearpage") %>

#+ echo=False
utilities.change_pweave_figure_size_heatmap(pdf_format)
visualizations.plot_heatmap(document,vars,samples,top_taxonomy_genera,top_data_genera,
    pdf_format,"Top {} genera by average abundance (Bray-Curtis)".format(max_sets_heatmap),max_sets_heatmap,method="lbraycurtis")
utilities.reset_pweave_figure_size()

#' ## Barplot

#' ### Species

#+ echo=False
top_taxonomy, top_data = utilities.top_rows(species_taxonomy, species_data, max_sets_barplot,
    function="average") 

sorted_data, sorted_samples = visualizations.sort_data(document, top_data, samples)

# add other to the taxonomy data, other represents the total abundance of all species not included in the top set
top_taxonomy, sorted_data = visualizations.fill_taxonomy_other(top_taxonomy, sorted_data)

#+ echo=False
# plot all samples taxonomy
document.plot_stacked_barchart(sorted_data, row_labels=top_taxonomy, 
    column_labels=sorted_samples, title="Top "+str(max_sets_barplot)+" species by average abundance",
    ylabel="Relative abundance", legend_title="Species", legend_style="italic")

#' Stacked barplot of <% print(max_sets_barplot) %> most abundant species among samples.
#' Samples in the plot were sorted on the species with the highest mean abundances among samples, in decreasing order. 

#+ echo=False
# plot a barchart for a set of categorical data
categorical_metadata = visualizations.plot_grouped_and_average_barplots_taxonomy(document, vars, sorted_samples, sorted_data, top_taxonomy, max_sets_barplot)

#' <% if categorical_metadata: print("Stacked barplot of species average abundance grouped by metadata.") %>

#' ### Genera

#+ echo=False
# get the top genera using the max set for the barplots
top_taxonomy_genera, top_data_genera = utilities.top_rows(genera_taxonomy, genera_data, max_sets_barplot,
    function="average")

sorted_data_genera, sorted_samples_genera = visualizations.sort_data(document, top_data_genera, samples)

# add the other category to reflect genera not included in top set
top_taxonomy_genera, sorted_data_genera = visualizations.fill_taxonomy_other(top_taxonomy_genera, sorted_data_genera)

# plot genera for all samples
document.plot_stacked_barchart(sorted_data_genera, row_labels=top_taxonomy_genera,
    column_labels=sorted_samples_genera, title="Top "+str(max_sets_barplot)+" genera by average abundance",
    ylabel="Relative abundance", legend_title="Genera", legend_style="italic")

#' Stacked barplot of <% print(max_sets_barplot) %> most abundant genera among samples.
#' Samples in the plot were sorted on the genera with the highest mean abundances among samples, in decreasing order.

#+ echo=False
# plot a barchart for a set of categorical data
categorical_metadata=visualizations.plot_grouped_and_average_barplots_taxonomy(document, vars, sorted_samples_genera, sorted_data_genera, top_taxonomy_genera, max_sets_barplot, feature="genera")

#' <% if categorical_metadata: print("Stacked barplot of genera average abundance grouped by metadata.") %>



#!/usr/bin/env python

import os
import argparse
import subprocess
import time
import shlex

GRID_JOBS_OPTION="--grid-jobs"

STARTUP_TIME = 5 * 60
MAX_SLEEP = 40 * 60

def process_ps_stdout(stdout):
    """ Process the stdout of the ps command """
    return [i.split()[0] for i in filter(lambda x: x, stdout.decode("utf-8").split("\n")[1:])]

def get_children(pid):
    """ Get the pids of the children of this pid """
    try:
        stdout=subprocess.check_output(["ps","--ppid",pid,"-o","pid"])
    except subprocess.CalledProcessError:
        stdout=[]

    pids=[]
    if stdout:
        pids=process_ps_stdout(stdout)

    return pids

def traverse_tree(pid,nodes):
    """ Get all of the children of the pid by walking the tree """

    for child in get_children(pid):
        nodes.update(traverse_tree(child,nodes))
    nodes.add(pid)

    return nodes

def get_pids(pid):
    """ Get the children and their children from parent pid """

    pids=set([pid])
    for child in get_children(pid):
        pids.update(traverse_tree(child,pids))
    
    return list(pids)

def main():

    # gather the arguments
    parser = argparse.ArgumentParser(
        description= "workflow burst wrapper",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "--workflow-command", 
        help="the workflow command to run \n[REQUIRED]", 
        metavar="<biobakery_workflows wmgx>", 
        required=True)
    parser.add_argument(
        "--max-sleep", 
        help="the total seconds to wait before terminating the workflow",
        type=int,
        default=MAX_SLEEP)

    args = parser.parse_args()

    # capture the total number of grid jobs
    command=shlex.split(args.workflow_command)
    try:
        grid_jobs = command[command.index(GRID_JOBS_OPTION)+1]
        sleep_time = int(grid_jobs) * 5 + STARTUP_TIME
        if sleep_time > MAX_SLEEP :
            sleep_time = MAX_SLEEP
    except ValueError, IndexError:
        print("Unable to determine grid jobs using default sleep setting")
        sleep_time = MAX_SLEEP

    # set max sleep if decided by the user
    if args.max_sleep != MAX_SLEEP:
        sleep_time = args.max_sleep

    # run the workflow and get the pid
    print("Starting workflow and then will wait for {} seconds".format(sleep_time))
    print(args.workflow_command)
    process=subprocess.Popen(args.workflow_command,shell=True)
    time.sleep(sleep_time)
    print("Stopping workflow. Please run '$ sacct' to track your submitted jobs.")
    # get all pids of children
    pids = get_pids(str(process.pid))
    process.kill()
    for id in pids:
        subprocess.call(["kill","-9",str(id)])

if __name__ == "__main__":
    main()



#!/usr/bin/env python

""" This script will merge fastq files with the 
    given string in the folder provided """

import sys

try:
    import argparse
except ImportError:
    sys.exit("Please upgrade to at least python v2.7")
    
import os
    
def catch_open(file,write=None):
    """ Try to open the file, return file handle, or exit if unable to open """
    try:
        if write:
            file_handle=open(file,"w")
        else:
            file_handle=open(file)
    except EnvironmentError:
        sys.exit("ERROR: Unable to open file: " + file)
        
    return file_handle

def write_file(infile,outfile):
    """ Append the outfile with the line in infile """
    
    with catch_open(infile) as file_handle:
        for line in file_handle:
            outfile.write(line)
    
def parse_arguments(args):
    """ Parse the arguments from the user"""
    
    parser=argparse.ArgumentParser(description="Merge fastq files.")
    parser.add_argument('input_folder',help="The input folder with the fastq files.")
    parser.add_argument('input_filenames',help="The search string for file names.")
    parser.add_argument('output_fastq',help="The merged fastq output file.")
    
    return parser.parse_args()

def main():
    # parse arguments
    args = parse_arguments(sys.argv)
    
    # get a list of all files in the input folder to merge
    input_merge_files=filter(lambda x: args.input_filenames in x, os.listdir(args.input_folder))

    # try to open the output file
    with catch_open(args.output_fastq, write=True) as file_handle_write:
        for file in input_merge_files:
            write_file(os.path.join(args.input_folder,file),file_handle_write)
 
if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Using the gene families or pathways files for a set of rna/dna samples,
# compute the normalization. This is the starr method.

import sys
import os
import argparse

def parse_arguments(args):
    """ 
    Parse the arguments from the user
    """
    parser = argparse.ArgumentParser(
        description= "RNA/DNA normalization\n",
        formatter_class=argparse.RawTextHelpFormatter,
        prog="rna_dna_norm")
    parser.add_argument(
        "--input-rna", 
        help="a tab-delimited abundance table of RNA data", 
        metavar="<input_rna.tsv>", 
        required=True)
    parser.add_argument(
        "--input-dna", 
        help="a tab-delimited abundance table of DNA data", 
        metavar="<input_dna.tsv>", 
        required=True)
    parser.add_argument(
        "--output", 
        help="a folder to write the output files", 
        metavar="<output_directory>", 
        required=True)
    parser.add_argument(
        "--mapping", 
        help="a tab-delimited mapping file of RNA to DNA samples (required if RNA/DNA sample names differ)", 
        metavar="<mapping.tsv>")
    parser.add_argument(
        "--min-abundance", 
        help="the minimum abundance value, data below this value are considered zero (default: 1)", 
        default=1)
    parser.add_argument(
        "--reduce-sample-name",
        help="remove the extra strings from the sample names",
        action='store_true')
    
    return parser.parse_args()
    
def read_table(file, min_abundance, reduce_sample_name, delimiter="\t"):
    """ Read the table from a text file with the first line the column
    names and the first column the row names. Filter humann2 un-identifiers and split
    data into stratified categories. """

    def format_data(x):
        """ Format the data as float, filtering based on min abundance """
        try:
            x = float(x)
        except ValueError:
            x = 0.0
            
        return x if x >= min_abundance else 0.0
    
    if reduce_sample_name:
        format_sample_name = lambda x: x.split("_Abundance")[0]
    else:
        format_sample_name = lambda x: x
    
    stratified_features={}
    unstratified_features={}
    strat_no_unclass_features={}
    
    stratified_data=[]
    unstratified_data=[]
    strat_no_unclass_data=[]
    
    stratified_index=0
    unstratified_index=0
    strat_no_unclass_index=0    
    
    with open(file) as file_handle:
        sample_names = {format_sample_name(name):i for i, name in enumerate(file_handle.readline().rstrip().split(delimiter)[1:])}
        for line in file_handle:
            line=line.rstrip().split(delimiter)
            if not "UNINTEGRATED" in line[0] and not "UNMAPPED" in line[0] and not "UNGROUPED" in line[0]:
                feature_name=line.pop(0)
                formatted_data=[format_data(i) for i in line]
                if "|" in feature_name:
                    # this is a stratified feature with species information
                    if not "|unclassified" in feature_name.lower():
                        strat_no_unclass_features[feature_name]=strat_no_unclass_index
                        strat_no_unclass_index+=1
                        strat_no_unclass_data.append(formatted_data)
                    stratified_features[feature_name]=stratified_index
                    stratified_index+=1
                    stratified_data.append(formatted_data)
                else:
                    unstratified_features[feature_name]=unstratified_index
                    unstratified_index+=1
                    unstratified_data.append(formatted_data)
                                    
    return ( unstratified_features, unstratified_data, stratified_features, 
             stratified_data, strat_no_unclass_features, strat_no_unclass_data,
             sample_names )

def read_mapping(file, delimiter="\t"):
    """ Read the mapping file """
        
    data=[]
    with open(file) as file_handle:
        for line in file_handle:
            data.append([item.strip() for item in line.rstrip().split(delimiter)])
                
    return data

def get_sample_feature(sample,feature,sample_labels,feature_labels,data):
    """ Return the data set for the sample, return zeros if name not found """
    
    # get sample index
    sample_index = sample_labels.get(sample,"NA")
    feature_index = feature_labels.get(feature,"NA")
    
    try:
        value=data[feature_index][sample_index]
    except (TypeError, IndexError):
        value=0
        
    return value
                
def divide_by_sample_total_abundance(data):
    """ For each sample in the data, divide all values by the total abundance 
    for the sample. """

    for j in range(len(data[0])):
        column_sum=sum(row[j] for row in data)
        if not column_sum == 0:
            for i in range(len(data)):
                data[i][j] = data[i][j] / column_sum
            
def compute_rna_dna_norm(samples, rna_features, rna_samples, rna_data,
    dna_features, dna_samples, dna_data):
    """ Divide the rna value by the corresponding dna value. First
    normalize by sample abundance. """

    # divide each value by the total abundance for the sample
    print("Normalize DNA")
    divide_by_sample_total_abundance(dna_data)
    print("Normalize RNA")
    divide_by_sample_total_abundance(rna_data)

    # compute norm
    norm_features=list(set(rna_features).union(dna_features))
    # create a list of empty lists, can't use [[]]*N because the empty list
    # are all pointers to the same list
    norm_data=[]
    for i in range(len(norm_features)):
        norm_data.append([])
        
    # compute the norm for all features for each sample
    for sample_name in samples:
        for index, feature in enumerate(norm_features):
            rna_value=get_sample_feature(sample_name,feature,rna_samples,rna_features,rna_data)
            dna_value=get_sample_feature(sample_name,feature,dna_samples,dna_features,dna_data)
            try:
                norm=rna_value/dna_value
            except ZeroDivisionError:
                if rna_value == 0:
                    norm="NaN"
                else:
                    norm="Inf"
            norm_data[index].append(norm)
        
    return norm_data, norm_features

def write_file(column_labels, row_labels, data, file):
    """ Write the data to a tab delimited file """
    
    # first order the rows alphabetically
    row_order=sorted(range(len(row_labels)),key=lambda i: row_labels[i])

    with open(file, "wb") as file_handle:
        file_handle.write("\t".join(column_labels)+"\n")
        for i in row_order:
            file_handle.write("\t".join([row_labels[i]]+[str(j) for j in data[i]])+"\n")
    
def main():
    # Parse arguments from command line
    args=parse_arguments(sys.argv)
    
    # read in the rna table, organizing by stratification
    print("Reading RNA table")
    ( rna_unstrat_features, rna_unstrat_data, rna_strat_features, rna_strat_data, 
      rna_strat_no_unclass_features, rna_strat_no_unclass_data, rna_samples ) = read_table(args.input_rna, args.min_abundance, args.reduce_sample_name)
    
    # read in the dna table, organizing by stratification
    print("Reading DNA table")
    ( dna_unstrat_features, dna_unstrat_data, dna_strat_features, dna_strat_data, 
      dna_strat_no_unclass_features, dna_strat_no_unclass_data, dna_samples ) = read_table(args.input_dna, args.min_abundance, args.reduce_sample_name)

    # apply sample map if provided
    if args.mapping:
        map_data = read_mapping(args.mapping)
        # convert the dna/rna sample names to the map name
        # map should be rna sample then dna sample names
        for data in map_data:
            # ignore comments or headers
            if not data[0].strip().startswith("#"):
                # find the name in the dna columns and replace
                try:
                    rna_name, dna_name = data
                except ValueError:
                    continue
                try:
                    dna_samples[rna_name]=dna_samples[dna_name]
                    del dna_samples[dna_name]
                except (ValueError, KeyError):
                    print("Mapping name "+dna_name+" is not included in the dna input file.")
    
    # get the intersection of the samples
    # apply list to set ordering
    samples=list(set(rna_samples).intersection(dna_samples))
    
    # check for matching sample names
    if len(samples) == 0:
        message="The rna/dna sample names do not match. "
        if args.mapping is None:
            message+=" Please provide a mapping file."  
        else:
            message+="Please check the formatting of the mapping file."
        sys.exit(message)
            
    # compute the norm for three sets of values: unstratified, all stratified, and
    # the stratified minus the unclassified set
    print("Compute unstratified features")
    norm_unstrat_data, norm_unstrat_features=compute_rna_dna_norm(samples, rna_unstrat_features, rna_samples,
        rna_unstrat_data, dna_unstrat_features, dna_samples, dna_unstrat_data)
    print("Compute stratified features")
    norm_strat_data, norm_strat_features=compute_rna_dna_norm(samples, rna_strat_features, rna_samples,
        rna_strat_data, dna_strat_features, dna_samples, dna_strat_data)
    print("Compute only classified features")
    norm_strat_no_unclass_data, norm_strat_no_unclass_features=compute_rna_dna_norm(samples,  
        rna_strat_no_unclass_features, rna_samples, rna_strat_no_unclass_data, 
        dna_strat_no_unclass_features, dna_samples, dna_strat_no_unclass_data)
    
    # create the output folder if needed
    if not os.path.isdir(args.output):
        print("Creating output directory: " + args.output)
        try:
            os.mkdir(args.output)
        except EnvironmentError:
            sys.exit("Unable to create output directory.")
    
    # write the output files
    output_unstrat_file=os.path.join(args.output,"rna_dna_relative_expression_unstratified.tsv")
    output_strat_file=os.path.join(args.output,"rna_dna_relative_expression.tsv")
    output_strat_no_unclass_file=os.path.join(args.output,"rna_dna_relative_expression_no_unclassifed.tsv")
    
    print("Writing unstratified table")
    write_file(["# features"]+samples, norm_unstrat_features, norm_unstrat_data,
        output_unstrat_file)
    print("Output file written: "+output_unstrat_file)
    
    print("Writing stratified table")
    write_file(["# features"]+samples, norm_unstrat_features+norm_strat_features,
        norm_unstrat_data+norm_strat_data, output_strat_file)
    print("Output file written: "+output_strat_file)
    
    print("Writing only classified table")
    write_file(["# features"]+samples, norm_unstrat_features+norm_strat_no_unclass_features,
        norm_unstrat_data+norm_strat_no_unclass_data, output_strat_no_unclass_file)
    print("Output file written: "+output_strat_no_unclass_file)
        
if __name__ == "__main__":
    main()
         
        



#!/usr/bin/env python

import sys
import os
import argparse

try:
    from biobakery_workflows import utilities
except ImportError:
    sys.exit("Please install biobakery workflows.")

# This script will trim a taxonomy file to only include the most specific clade
# followed by any unknown taxonomy if included in the full name.

def parse_arguments(args):
    """ 
    Parse the arguments from the user
    """
    parser = argparse.ArgumentParser(
        description= "Trim taxonomy\n",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "-i", "--input",
        help="the otu table\n[REQUIRED]",
        metavar="<otu.tsv>",
        required=True)
    parser.add_argument(
        "-o", "--output",
        help="file to write the trimmed otu table\n[REQUIRED]",
        metavar="<output.tsv>",
        required=True)
    parser.add_argument(
        "-t", "--taxonomy-column",
        help="the column number, zero based index, containing taxonomy information [auto-detected]",
        default=None,
        type=int)

    return parser.parse_args()


def main():

    args=parse_arguments(sys)

    try:
        file_handle_write=open(args.output,"wt")
    except EnvironmentError:
        sys.exit("Error: Unable to open output file: " + args.output)

    try:
        file_handle_read=open(args.input,"rt")
    except EnvironmentError:
        sys.exit("Error: Unable to read input file: " + args.input)

    # write the header to the new file
    file_handle_write.write(file_handle_read.readline())
    
    # trim the taxonomy
    for line in file_handle_read:
        # ignore lines that are comments
        if line.startswith("#"):
            file_handle_write.write(line)
        else:
            data=line.rstrip().split("\t")
            if args.taxonomy_column is None:
                # try to figure out which column has the taxonomy data
                try:
                    args.taxonomy_column=[index for index, value in enumerate(data) if "k__" in value][0]
                except IndexError:
                    sys.exit("Error unable to find the taxonomy column. Please provide it with the option --taxonomy-column <0>.")
            data[args.taxonomy_column]=utilities.taxonomy_trim([data[args.taxonomy_column]])[0]
            file_handle_write.write("\t".join(data)+"\n")

    file_handle_read.close()
    file_handle_write.close()

if __name__ == "__main__":
    main()





"""
bioBakery Workflows: visualizations module
A collection of utilities for the document visualizations

Copyright (c) 2017 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os
import copy
import sys
import subprocess
import tempfile
import shutil

from . import utilities

def plot_grouped_and_average_barplots_taxonomy(document, vars, sorted_samples, sorted_data, top_taxonomy,
    max_sets_barplot, feature="species", sort_by_name=False, sort_by_name_inverse=False, ylabel="Relative abundance"):
    """ Plot grouped barplots and average barplots for all of the features provided.

        Args:
            document (anadama2.document): Document to use to add plots
            vars (dict): The dictionary of input variables provided for the visualization run
            sorted_samples (list): The sample names organized to match the data
            sorted_data (list): The data organized to match the sample/taxonomy
            top_taxonomy (list): The list of full taxonomy names
            max_sets_barplot (int): The max number of features (default species) to plot
            feature (string): The data being plotted (default species)
            sort_by_name (bool): If set, sort data by sample name instead of abundance
            sort_by_name_inverse (bool): If true, sort by the inverse of the name (so the reverse of the string)
                this is useful for samples with sample name plus features
            ylabel (string): The label for the y-axis for the average plots

        Returns: 
            list: A list of metadata strings (one for each categorical feature plotted)
    """

    # plot a barchart for a set of categorical data
    categorical_metadata = []
    if metadata_provided(vars):
        categorical_metadata, ordered_sorted_data, ordered_metadata, samples_found = merge_categorical_metadata(vars, sorted_samples,
            sorted_data)
        for cat_metadata in categorical_metadata:
            plot_grouped_taxonomy_subsets(document, ordered_sorted_data, cat_metadata, top_taxonomy,
                samples_found,title="Top {} {} by average abundance".format(max_sets_barplot,feature),ylabel=ylabel,sort_by_name=sort_by_name,
                sort_by_name_inverse=sort_by_name_inverse)
        # plot average for all samples grouped by categorical metadata
        for cat_metadata in categorical_metadata:
            plot_average_taxonomy(document, ordered_sorted_data, samples_found, top_taxonomy, cat_metadata, max_sets_barplot, legend_title=feature, ylabel=ylabel)

    return categorical_metadata

def fill_taxonomy_other(top_taxonomy, sorted_data):
    """ Fill the taxonomy abundances provided with 'other' so each totals 100.

        Args:
            top_taxonomy (list): The list of full taxonomy names
            sorted_data (list): The data sorted by abundance matched to the taxonomy list

        Returns:
            list : The list of full taxonomy names plus 'other'
            list : The list of data filled with the 'other' abundance
    """

    import numpy

    # add other to the taxonomy data
    # other represents the total abundance of all species not included in the top set
    new_top_taxonomy = top_taxonomy + ["other"]
    new_sorted_data = sorted_data
    other_abundances=[]
    for column in numpy.transpose(sorted_data):
        other_abundances.append(100-sum(column))

    if isinstance(new_sorted_data,numpy.ndarray):
        new_sorted_data = new_sorted_data.tolist()
    new_sorted_data.append(other_abundances)

    return new_top_taxonomy, new_sorted_data


def show_pcoa_metadata(document, vars, samples, top_taxonomy, pcoa_data, title):
    """ Plot pcoa for each feature if metadata has been provided

        Args:
            document (anadama2.document): Document to use to add plots
            vars (dict): The dictionary of input variables provided for the visualization run
            samples (list): The sample names organized to match the data
            top_taxonomy (list): The full taxonomic names organized to match the data
            pcoa_data (list): The data (range 0 to 1) organized to match the samples and taxonomy
            title (string): The base string for the title for the plots
 
        Return: None

    """

    if 'metadata' in vars and vars['metadata']:
        # organize metadata for plot if available
        sample_metadata=vars["metadata"][0]
        for category in vars["metadata"][1:]:
            name=category[0]
            metadata_mapping=dict((x,y) for x,y in zip(sample_metadata[1:],category[1:]))
            metadata_dict = vars['metadata_labels']
            metadata_type = metadata_dict[name]
            
            document.show_pcoa(samples, top_taxonomy, pcoa_data,title=title+" - "+name,
                               metadata=metadata_mapping, metadata_type=metadata_type)

def get_top_taxonomy_by_level(taxonomy, samples, relab_data, max_taxa, taxa_level=5):
    """ Get the top, based on average abundance, set of taxa at the genus level

        Args:
            taxonomy (list): The full taxonomic names organized to match the data
            samples (list): The sample names organized to match the data
            relab_data (list): The relative abundance data
            max_taxa (int): The max number of genera to identify
            taxa_level (int): The taxonomy level to select (default of 5 is genus, using 4 would be family)

        Return:
            list : The sorted sample names (sorted to list the top genera first)
	    list : The data filtered to only the top genera (sorted by decreasing abundance)
            list : The data filtered to only the top genera 
            list : The taxonomic names (short for plotting) of the top taxa
            int : The recommended font size for the ploting legend (based on the taxonomic names)
    """ 

    import numpy

    # get the taxa summarized by genus level
    genus_level_taxa, genus_level_data = utilities.taxa_by_level(taxonomy, relab_data, level=taxa_level)
    # get the top rows of the relative abundance data
    top_taxa, top_data = utilities.top_rows(genus_level_taxa, genus_level_data, max_taxa, function="average")
    # shorten the top taxa names to just the genus level for plotting
    top_taxa_short_names = utilities.taxa_shorten_name(top_taxa, level=5, remove_identifier=True)
    # check for duplicate genera in list
    legend_size = 7
    if len(top_taxa_short_names) != len(list(set(top_taxa_short_names))):
        # if duplicate names, then add family to the taxonomy
        top_taxa_short_names = [family+"."+genus for family, genus in zip(utilities.taxa_shorten_name(top_taxa, level=taxa_level-1),utilities.taxa_shorten_name(top_taxa, level=taxa_level))]
        # reduce legend size to fit names
        legend_size = 5

    # sort the data so those with the top genera are shown first
    sorted_samples, sorted_data = utilities.sort_data(top_data[0], samples)
    transpose_top_data = numpy.transpose(top_data)
    sorted_top_data = numpy.transpose([transpose_top_data[samples.index(sample)] for sample in sorted_samples])

    return sorted_samples, sorted_top_data, top_data, top_taxa_short_names, legend_size


def metadata_provided(vars):
    """ Check if metadata was provided by the user for this visualization run 

        Args:
            vars (dict): The dictionary of input variables provided for the visualization run 

        Returns:
            bool : True if metadata was provided from the user for this run

    """

    if 'metadata' in vars and vars['metadata'] and 'metadata_labels' in vars and vars['metadata_labels']:
        return True
    else:
        return False

def merge_categorical_metadata(vars, sorted_samples, sorted_data):
    """ Merge data with metadata and filter continuous data 

        Args:
            vars (dict): The dictionary of input variables provided for the visualization run
            sorted_samples (list): A list of sample names sorted to match the data provided
            sorted_data (list): The full set of data (taxonomy or other)

        Returns: 
            list: A list of metadata strings (one for each categorical feature)
            list: The data modified to only include those samples with metadata
            list: The metadata sorted to match the order of the data (categorical and continuous)
            list: The metadata sorted to match the order of the data (only categorical)
    """

    # get the metadata organized into the same sample columns as the data
    new_data, samples_found = utilities.merge_metadata(vars['metadata'], sorted_samples, sorted_data, values_without_names=True)
    # split the data and metadata
    ordered_metadata=new_data[0:len(vars['metadata'])-1]
    ordered_sorted_data=new_data[len(vars['metadata'])-1:]
    # get the categorical metadata
    categorical_metadata=utilities.filter_metadata_categorical(ordered_metadata, vars['metadata_labels'])

    return categorical_metadata, ordered_sorted_data, ordered_metadata, samples_found


def plot_average_taxonomy(document, ordered_sorted_data, samples_found, top_taxonomy, cat_metadata, max_sets_barplot, legend_title, ylabel="Relative Abundance"):
    """ Plot the average taxonomy sorted by species abundance for a single feature.

        Args:
            document (anadama2.document): Document to use to add plots
            ordered_sorted_data (list): A list of sorted/ordered data (to match the sample names and metadata)
            samples_found (list): An ordered list of sample names (to match the data/metadata)
            top_taxonomy (list): A list of the taxonomy names ordered to match the data
            cat_metadata (list): The ordered categorical metadata (for a single feature)
            max_sets_barplot (int): The max number of taxonomic values (to use for the title)
            legend_title (string): The legend title for the plots
            ylabel (string): The label for the y-axis

        Returns: None
    """

    # group the samples by metadata
    sorted_data_grouped, sorted_samples_grouped = utilities.group_samples_by_metadata(cat_metadata, ordered_sorted_data, samples_found)
    metadata_names=[]
    average_data=[]
    for name, row in sorted_data_grouped.items():
        metadata_names.append(name)
        average_data.append([sum(group)/(1.0*len(group)) for group in row])

    # reorder average data so it is grouped by taxonomy
    average_data = zip(*average_data)

    # sort the data by abundance or name (depending on metadata type, use name if numeric)
    sort_by_name=False
    if document.sorted_data_numerical_or_alphabetical(metadata_names) != sorted(metadata_names):
        sort_by_name = True

    sorted_data, sorted_names = sort_data(document, average_data, metadata_names, sort_by_name=sort_by_name)

    document.plot_stacked_barchart(sorted_data, row_labels=top_taxonomy,
        column_labels=sorted_names, 
        title="Top {} {} group average - {}".format(max_sets_barplot, legend_title, cat_metadata[0]),
        ylabel=ylabel, legend_title=legend_title, legend_style="italic")

def sort_data(document, top_data, samples, sort_by_name=False, sort_by_name_inverse=False):
    # sort the top data so it is ordered with the top sample/abundance first
    if sort_by_name:
        sorted_sample_indexes=[samples.index(a) for a in document.sorted_data_numerical_or_alphabetical(samples)]
    elif sort_by_name_inverse:
        inverse_samples = [sample[::-1] for sample in samples]
        sorted_sample_indexes=[inverse_samples.index(a) for a in document.sorted_data_numerical_or_alphabetical(inverse_samples)]
    else:
        sorted_sample_indexes=sorted(range(len(samples)),key=lambda i: top_data[0][i],reverse=True)

    sorted_samples=[samples[i] for i in sorted_sample_indexes]
    sorted_data=[]
    for row in top_data:
        sorted_data.append([row[i] for i in sorted_sample_indexes])
    return sorted_data, sorted_samples

def plot_grouped_taxonomy_subsets(document, sorted_data, cat_metadata, top_taxonomy, samples_found, title, 
    ylabel="Relative abundance", legend_title="Species", legend_size=7, max_subsets=2, sort_by_name=False,
    sort_by_name_inverse=False):
    """ Plot the grouped taxonomy with samples sorted by species abundance for each feature.

        Args:
            document (anadama2.document): Document to use to add plots
            sorted_data (list): A list of sorted/ordered data (to match the sample names and metadata)
            cat_metadata (list): A list of metadata (feature) names
            top_taxonomy (list): A list of the taxonomy names ordered to match the data
            samples_found (list): An ordered list of sample names (to match the data/metadata)
            title (string): The base string for the title for all plots
            ylabel (string): The label for the y-axis
            legend_title (string): The legend title for the plots
            legend_size (int): The font size for the legend
            max_subsets (int): The max number of subsets to use for each plot
            sort_by_name (bool): If set, then sort by sample name instead of abundance
            sort_by_name_inverse (bool): If true, sort by the inverse of the name (so the reverse of the string)
                this is useful for samples with sample name plus features

        Return: None
    """

    # group the samples by metadata
    sorted_data_grouped, sorted_samples_grouped = utilities.group_samples_by_metadata(cat_metadata, sorted_data, samples_found)
    # sort the data by abundance or sample name
    for metadata_type in sorted_data_grouped:
        sorted_data_grouped[metadata_type], sorted_samples_grouped[metadata_type] = sort_data(document, sorted_data_grouped[metadata_type], sorted_samples_grouped[metadata_type], sort_by_name=sort_by_name, sort_by_name_inverse=sort_by_name_inverse)

    # print out a plot for each group of metadata
    sorted_metadata_subsets=document.sorted_data_numerical_or_alphabetical(sorted_data_grouped.keys())
   
    # split into subsets
    split_sorted_metadata_subsets = [sorted_metadata_subsets[x:x+max_subsets] for x in range(0, len(sorted_metadata_subsets), max_subsets)]
   
    # make sure the last group is not just a single data set
    if len(split_sorted_metadata_subsets[-1]) == 1:
        last_set = split_sorted_metadata_subsets.pop()
        split_sorted_metadata_subsets[-1].append(last_set[0])

    for metadata_subset in split_sorted_metadata_subsets:
        subset_sorted_data_grouped=dict((key, sorted_data_grouped[key]) for key in metadata_subset)
        subset_sorted_samples_grouped=dict((key, sorted_samples_grouped[key]) for key in metadata_subset)

        # get title addition for subset
        title_add=""
        if len(sorted_metadata_subsets) > max_subsets:
            title_add=" "+metadata_subset[0]+" to "+metadata_subset[-1]

        document.plot_stacked_barchart_grouped(subset_sorted_data_grouped, row_labels=top_taxonomy,
            column_labels_grouped=subset_sorted_samples_grouped, title=title+" - "+str(cat_metadata[0])+title_add,
            ylabel=ylabel, legend_title=legend_title, legend_style="italic", legend_size=legend_size)

def plot_heatmap(document,vars,samples,top_taxonomy,top_data,pdf_format,title=None,max_sets_heatmap=25,method="correlation"):
    """ Generate a heatmap using the doc function. Include metadata if available. """

    # set the default title if not provided
    if not title:
        title = "Top {} species by average abundance".format(max_sets_heatmap)

    # if there is metadata, add it to the top taxonomy data
    if 'metadata' in vars and vars['metadata']:
        merged_data, metadata_samples=utilities.merge_metadata(vars['metadata'], samples,
            [[top_taxonomy[i]]+top_data[i] for i in range(len(top_taxonomy))])
        metadata_taxonomy=[row.pop(0) for row in merged_data]
        # get the metadata row numbers
        metadata_rows=range(1,len(vars['metadata']))
        document.show_hclust2(metadata_samples, metadata_taxonomy, merged_data,
            title=title, metadata_rows=metadata_rows, method=method)
    else:
        document.show_hclust2(samples,top_taxonomy,top_data,title=title,method=method)


def plot_hallagram(document, feature_set_1_data, feature_set_2_data, axis1_label, axis2_label, model=None, title=None,
    output_folder=None, strongest=10, show_table=False,q_value=0.1):
    """ Run halla on the two feature sets and plot the hallagram with the strongest associations 
        The first data line for each feature set should be a header of sample ids
        Requires halla v0.8.7
        If show_table is set, then instead of including heatmap include a table of associations
    """

    from matplotlib._png import read_png
    import matplotlib.pyplot as pyplot

    # create a temp output folder, if not provided
    if output_folder:
        outfolder = output_folder
        # if folder does not exist, then create
        if not os.path.isdir(outfolder):
            os.makedirs(outfolder)
    else:
        outfolder=tempfile.mkdtemp(suffix="biobakery_workflows_halla",dir=output_folder)

    # write the lines for the two feature set files
    feature_set_1 = os.path.join(outfolder,"feature1.tsv")
    feature_set_2 = os.path.join(outfolder,"feature2.tsv")
    for lines, file_name in [[feature_set_1_data,feature_set_1],[feature_set_2_data,feature_set_2]]:
        with open(file_name,"w") as file_handle:
            file_handle.write("\n".join(["\t".join(map(str,l)) for l in lines]))

    # run halla
    halla_command = ["halla","-X", feature_set_1, "-Y", feature_set_2,"--output", outfolder, "--header"]
    if model:
        halla_command += ["-m",model]
    try:
        subprocess.check_call(halla_command)
    except subprocess.CalledProcessError:
        print("Error: Unable to run halla")
    
    if show_table:
        # display the table of associations instead of including the heatmap
        associations_file = os.path.join(outfolder,"associations.txt")
        # check that the file is not empty
        with open(associations_file) as file_handle:
            associations_total = len(file_handle.readlines())-1

        if associations_total > 0:
            columns, row_names, halla_data = document.read_table(associations_file, format_data=str)

            # reduce data to only those columns to display in the table
            cluster_results = [[row[0].replace(";","\n"),row[2].replace(";","\n"),"{0:.7f}".format(float(row[4])),"{0:.3f}".format(float(row[5]))] for row in halla_data]
            # filter any results that are more than the max q_value
            cluster_results = list(filter(lambda row: float(row[-1]) < q_value, cluster_results))
        
            row_labels = list(map(lambda x: str(x)+"   ",range(1,len(cluster_results)+1)))
            column_labels = [axis1_label, axis2_label,"  p-value  ","q-value"]
            document.show_table(cluster_results, row_labels, column_labels, title, font=6)
        else:
            print("No associations found for "+title)
    else:
        # run hallagram
        output_png = os.path.join(outfolder,"hallagram.png")
        hallagram_command = ["hallagram",os.path.join(outfolder,"similarity_table.txt"),
            os.path.join(outfolder,"hypotheses_tree.txt"), os.path.join(outfolder,"associations.txt"),
            "--outfile",output_png,"--strongest",str(strongest),"--axlabels",axis1_label,axis2_label]
        if model:
            hallagram_command += ["--similarity",model]
        try:
            subprocess.check_call(hallagram_command)
        except subprocess.CalledProcessError:
            print("Error: Unable to run hallagram")

        if os.path.isfile(output_png):
            hallagram_png=read_png(output_png)        

            # create a subplot and remove the frame and axis labels
            fig = pyplot.figure()
            subplot = fig.add_subplot(111, frame_on=False)
            subplot.xaxis.set_visible(False)
            subplot.yaxis.set_visible(False)
        
            # set title if provided
            if title:
                fig.suptitle(title, fontsize=16)

            # show but do not interpolate (as this will make the text hard to read)
            try:
                pyplot.imshow(hallagram_png, interpolation="none")
            except TypeError:
                print("Unable to display hallagram plot")
                pass

            # this is needed to increase the image size (to fit in the increased figure)
            pyplot.tight_layout()

    # remove the temp folder
    if not output_folder:
        shutil.rmtree(outfolder) 

def plot_pcoa_top_average_abundance(document, samples, feature_names, feature_data, feature_type, scale_data=None, legend_title="% Abundance", max_sets=6):
    """ Plot multiple pcoa in a single figure for the top abundances for the feature set """

    # if function is provided, scale the abundance data
    if scale_data:
        new_data = []
        for row in feature_data:
            new_data.append(map(scale_data,row))
        feature_data = new_data

    # get the top features by average abundance
    top_names, top_data = utilities.top_rows(feature_names, feature_data, max_sets, function="average")

    top_abundances=dict((x,y) for x,y in zip(["#"+str(i+1)+" "+x for i,x in enumerate(top_names[:max_sets])], top_data[:max_sets]))
    document.show_pcoa_multiple_plots(samples, feature_names, feature_data,
        "PCoA Ordination of "+feature_type+", top "+str(max_sets)+" "+feature_type+" by average abundance", top_abundances, legend_title)
    
def qc_read_counts(document, file):
    """ Read in the file of read counts compiled from kneaddata logs with the utility script """
    
    columns, samples, data = document.read_table(file, format_data=int)
    
    # shorten known database names
    columns=[name.replace("SILVA_128_LSUParc_SSUParc_ribosomal_RNA","rRNA") for name in columns]
    columns=[name.replace("Homo_sapiens","hg38") for name in columns]
    columns=[name.replace("human_hg38_refMrna","mRNA") for name in columns]
    
    # change the names of the raw and trimmed columns (to include case)
    columns=[name.replace("raw","Raw").replace("trimmed","Trim").replace("decontaminated","") for name in columns]
    
    # check if this is single or paired end data
    if list(filter(lambda x: "single" in x, columns)):
        # remove single from the column name
        columns=[name.replace("single","").strip() for name in columns[:-1]]
        
        # return all but the final filtered column as this is not needed
        data=[row[:-1] for row in data]
        
    else:
        # remove the final columns from the list as these are not needed
        columns=list(filter(lambda x: not "final" in x, columns))
        
        # organize the data into pairs and orphans
        pairs_index = [index for index, name in enumerate(columns) if "pair1" in name]
        pairs_columns = [name.replace("pair1","").strip() for index, name in enumerate(columns) if index in pairs_index]
        orphans_index = [index for index, name in enumerate(columns) if "orphan" in name]
        orphans_columns = [name for index, name in enumerate(columns) if index in orphans_index]
        
        pairs_data=[]
        orphans_data=[]
        for row in data:
            pairs_data.append([])
            orphans_data.append([])
            for index in range(len(row)):
                if index in pairs_index:
                    pairs_data[-1].append(row[index])
                if index in orphans_index:
                    orphans_data[-1].append(row[index])
                
        columns = pairs_columns, orphans_columns
        data = pairs_data, orphans_data
    
    return columns, samples, data
    

def feature_counts(document, read_counts_file, feature_counts_file):
    """ Compute feature counts from the humann2 log read counts file and the feature counts file """
    
    # read in the read count and feature count files
    read_type, read_samples, read_count_data = document.read_table(read_counts_file)
    feature_type, feature_samples, feature_count_data = document.read_table(feature_counts_file)
    
    # remove any samples for which the prescreen did not find any species so nucleotide search was bypassed
    # these samples will have NA read counts but could have non-zero species count (species is the last column)
    read_samples, read_count_data = utilities.filter_zero_rows(read_samples, read_count_data, ignore_index=-1)
    
    # get the total reads for samples along with those for nucleotide alignment and translated
    # convert values to log10

    total_reads=[utilities.try_log10(row[read_type.index("total reads")]) for row in read_count_data]
    nucleotide_reads=[utilities.try_log10(row[read_type.index("total nucleotide aligned")]) for row in read_count_data]
    translated_reads=[utilities.try_log10(row[read_type.index("total translated aligned")]) for row in read_count_data]
    
    # sort the feature counts so they are in the same sample order as the read counts
    all_feature_counts={sample:row for sample, row in zip(feature_samples, feature_count_data)}
    sorted_feature_count_data=[all_feature_counts[sample] for sample in read_samples]
    
    # get the counts by each feature type
    # convert values to log10
    genefamilies_counts=[utilities.try_log10(row[feature_type.index("humann2_genefamilies_relab_counts")]) for row in sorted_feature_count_data]
    ecs_counts=[utilities.try_log10(row[feature_type.index("humann2_ecs_relab_counts")]) for row in sorted_feature_count_data]
    pathabundance_counts=[utilities.try_log10(row[feature_type.index("humann2_pathabundance_relab_counts")]) for row in sorted_feature_count_data]
    
    return total_reads, nucleotide_reads, translated_reads, genefamilies_counts, ecs_counts, pathabundance_counts

def write_pathway_average_variance_table(document, file_name, data, names_and_descriptions, format_table_decimal="{:.3}"):
    """ Write a file of the pathways including averages and variance """

    # get the average abundances and descriptions for the pathways
    top_average_pathways_file = os.path.join(document.data_folder, file_name)
    
    # get the average abundances, formatting as a single value per row 
    average_abundance_variance=[]
    for average, variance in zip(utilities.row_average(data), utilities.row_variance(data)):
        average_abundance_variance.append([format_table_decimal.format(average),format_table_decimal.format(variance)])
    
    document.write_table(["# Pathway","Average abundance", "Variance"], names_and_descriptions, 
                         average_abundance_variance, top_average_pathways_file)
    
    return average_abundance_variance

def top_average_pathways(document, file, max_sets):
    """ Read the pathways file and get the top average pathways """
    
    # read in the samples and get the data with out the stratification by bug
    samples, pathways, data = document.read_table(file)
    pathway_names = utilities.pathway_names(pathways)
    pathways, data = utilities.remove_stratified_pathways(pathways, 
        data, remove_description=True)
    
    # remove extra identifier from sample name if included in workflow
    samples = [sample.replace("_Abundance","").replace("-RPKs","") for sample in samples]
    
    # get the average abundance for the pathways
    top_pathways, top_data = utilities.top_rows(pathways,
        data, max_sets, function="average")
    
    # get the top names with descriptions
    top_names_and_descriptions = [name+":"+pathway_names[name] for name in top_pathways]
    
    return samples, top_pathways, top_data, top_names_and_descriptions

def show_table_max_rows(document, data, row_labels, column_labels, title, table_file,
    max_rows=20, format_data_comma=None, location="center", font=None, max_columns=7):
    """ For large numbers of samples, only show a reduced table """
    
    table_message="A data file exists of this table: "
    large_table_message="The table is too large to include the full table in this document."+\
        " A partial table is shown which includes only {max} {item}."+\
        " Please see the data file for the full table: "
        
    # check if there are too many rows
    partial_table_rows=False
    if len(row_labels) > max_rows:
        data=data[:max_rows]
        row_labels=row_labels[:max_rows]
        partial_table_rows=True
        
    # check if there are too many columns
    partial_table_columns=False
    if len(column_labels) > max_columns-1:
        data=[row[:max_columns-1] for row in data]
        column_labels=column_labels[:max_columns-1]
        partial_table_columns=True
        
    # determine the message and title if this is a full or partial table
    if partial_table_rows or partial_table_columns:
        if partial_table_rows:
            message=large_table_message.format(max=max_rows,item="rows")
        else:
            message=large_table_message.format(max=max_columns,item="columns")
        title+=" (partial table)"
    else:
        message=table_message
        
    # render the table
    document.show_table(data, row_labels, column_labels, 
        title, format_data_comma=format_data_comma, location=location, font=font)
    
    message+="[{file}](data/{file})".format(file=os.path.basename(table_file))
        
    return message

def print_pathways_urls(names, descriptions, total):
    """ List pathways with urls, including descriptions """
    
    print("Detailed functions of the top {} pathways can be found on the following MetaCyc pages:  ".format(total))
    
    print("")
    for pathway, desc in zip(names[:total], descriptions[:total]):
        print(" * ["+desc+"]("+utilities.metacyc_url(pathway)+")  ")
        
    print("")
    print("To learn more about other pathways, search for the pathway by name on the [MetaCyc](https://metacyc.org/) website.")

class Workflow(object):
    
    @classmethod
    def format_caption(cls,name,**keywords):
        return cls.captions[name].format(**keywords)


class ShotGun(Workflow):
    captions={}
    
    # add captions for functional data section
    captions["functional_intro"]="This report section contains preliminary "+\
        "exploratory figures that summarize HUMAnN2 functional profiling of "+\
        "all samples. HUMAnN2 performs species-specific and species-agnostic "+\
        " quantification of gene families, EC enzyme modules, and pathways, "+\
        "using the UniRef and MetaCyc databases. For more information on "+\
        "functional profiling and the databases used, see websites for "+\
        "[HUMAnN2](http://huttenhower.sph.harvard.edu/humann2), "+\
        "[UniRef](http://www.uniprot.org/help/uniref), "+\
        "and [MetaCyc](https://metacyc.org/)."
        
    captions["heatmap_intro"]="Hierarchical clustering of samples "+\
        "and {type}, using top {max_sets} {type} "+\
        "with highest mean relative abundance among samples. "+\
        "The 'average linkage' clustering on the Euclidean "+\
        "distance metric was used to cluster samples. The {type} "+\
        "dendrogram is based on pairwise ( {method} ) correlation between pathways. "+\
        "Samples are columns and pathway are rows. The heatmaps were generated "+\
        "with [Hclust2](https://bitbucket.org/nsegata/hclust2)."
        
    captions["feature_detection"]="Feature detection as a function of sequencing "+\
        "depth. Effect of sample sequencing depth on the ability to detect "+\
        "microbiome functional features in {seq_type} sequence data. HUMAnN2 "+\
        "functional profiling of {seq_short_type} quality filtered reads was performed on "+\
        "individual samples in species-specific mode (blue), i.e. nucleotide "+\
        "alignment against pangenomes of species identified in the sample "+\
        "with MetaPhlAn2, and in combined species-specific and -agnostic "+\
        "(orange) mode, in which reads not matching any pangenome reference "+\
        "sequences were subjected to translated searching against the "+\
        "UniRef90 database. Each profiled sample is represented by a "+\
        "orange and blue point in each plot. Linear regression fit is "+\
        "represented by straight lines in each plot."
        
    captions["pathway_abundance_heatmap"]="Abundances were {norm} transformed "+\
        "prior to clustering. The color bar represents relative abundances on a {norm} scale."  
        
    captions["scatter_reads_aligned"]="Number of aligned reads in species-specific "+\
        "(nucleotide search) and species-agnostic (translated search) HUMAnN2 mode "+\
        "as a function of input reads."
        
    captions["scatter_features"]="Detection of UniRef90 gene families, enzyme modules,"+\
        " and pathways as a function of aligned reads."
        
    captions["microbial_ratios"]="Proportion of reads remaining after removing host"+\
        " reads relative to the number of: i) quality-trimmed reads, and ii) raw "+\
        "unfiltered reads."
        
    captions["qc_intro"]="This report section contains information about the "+\
        "quality control processing for all {total_samples} {seq_type} fastq input "+\
        "files. These files were run through the "+\
        "[KneadData](http://huttenhower.sph.harvard.edu/kneaddata) QC pipeline. "+\
        "Reads were first trimmed then filtered against contaminate reference database{dbs}. "
    
    captions["qc_intro_multiple_db"]="Reads were filtered sequentially "+\
        "with those reads passing the first filtering step used as input to the next "+\
        "filtering step. This chain of filtering removes reads from all references in serial."
        
    captions["qc_intro_paired"]="\n \nData is organized by paired "+\
        "and orphan reads. When one read in a pair passes a filtering step and "+\
        "the other does not the surviving read is an orphan."
        
    captions["qc_intro_table"]="\nThe tables and plots are annotated as follows:\n \n"+\
        " * raw : Untouched fastq reads.\n"+\
        " * trim : Number of reads remaining after trimming bases with Phred score < 20. If the "+\
        "trimmed reads is < 50% of original length then it is removed altogether.\n"
        
    # set descriptions for command qc databases
    captions["qc_databases"]={}
    captions["qc_databases"]["hg38"]="The human genome database is used to remove "+\
        "reads originating from the host DNA."
    captions["qc_databases"]["mRNA"]="The human transcriptome (hg38 mRNA) database "+\
        "is used to remove reads originating from host gene isoforms."
    captions["qc_databases"]["rRNA"]="The SILVA (rRNA) database is used to remove "+\
        " small and large subunit ribosomal RNA."
    
    @classmethod
    def print_qc_intro_caption(cls, total_samples, databases, paired=None):
        """ Generate the qc intro caption based on the samples and databases """
        
        caption=cls.captions["qc_intro"]
        
        # if there are multiple databases, add the description about sequential filtering
        if len(databases) > 1:
            caption+=cls.captions["qc_intro_multiple_db"]
        
        if paired:
            caption+=cls.captions["qc_intro_paired"]
        caption+=cls.captions["qc_intro_table"]
        
        # add each database to the list
        dbs_list=[]
        for db in databases:
            dbs_list.append(db)
            desc=cls.captions["qc_databases"].get(db,"")
            caption+=" * {db} : Number of reads after depleting against reference database {list}. {desc}\n".format(
                db=db,list=" and ".join(dbs_list), desc=desc)
        caption+=" \n"
        
        # get the sequence type string
        seq_type="single-end"
        if paired:
            seq_type="paired-end"
        
        # format the caption to include the specific details for this data set
        caption=caption.format(total_samples=total_samples, seq_type=seq_type,
            dbs="s: "+", ".join(databases[:-1])+" and "+databases[-1] if len(databases) > 1 else " "+databases[0])
        
        for line in caption.split("\n"):
            print(line)
        

class Sixteen_S(Workflow):
    captions={}

    captions["itsintro"]='Implementing ITS pipeline for resolving sequence variants from ITS region\
         of paired-end sequencing reads, adopting the tutorial from\n\n \
         https://benjjneb.github.io/dada2/ITS_workflow.html \n \
         https://benjjneb.github.io/dada2/tutorial.html and \n \
         https://benjjneb.github.io/dada2/bigdata_paired.html \n\n with minor adjustments. \n \
         "Unlike the 16S rRNA gene, the ITS region is highly variable in length. The commonly amplified ITS1 and ITS2 regions \
         range from 200 - 600 bp in length. This length variation is biological, not technical, and arises from the high rates\
         of insertions and deletions in the evolution of this less conserved gene region. The length variation of the ITS region has \
         significant consequences for the filtering and trimming steps of the standard DADA2 workflow. First, truncation to a fixed \
         length is no longer appropriate, as that approach remove real ITS variants with lengths shorter than the truncation length.\
         Second, primer removal is complicated by the possibility of some, but not all, reads extending into the opposite primer when\
         the amplified ITS region is shorter than the read length." [ITS Tutorial] \n \
         "Critical addition to ITS workflows is the removal of primers on the forward and reverse reads, in a way that accounts\
         for the possibility of read-through into the opposite primer." [ITS Tutorial] \n \
         "Cutadapt" tool is used  for removal of primers from the ITS amplicon sequencing data. After initial step of primers \
         removal, the rest of ITS workflow matches DADA2 workflow.\n \
         Database UNITE is used for taxonomic assignment.\n\n '


    captions["dada2intro"]="Implementing DADA2 pipeline for resolving sequence variants from 16S rRNA \
        gene amplicon paired-end sequencing reads, adopting the tutorial from\n\n \
         https://benjjneb.github.io/dada2/tutorial.html and \n \
         https://benjjneb.github.io/dada2/bigdata_paired.html \n\n with minor adjustments.\
        \n\nThis report captures all the workflow steps necessary to reproduce the analysis. Notes and descriptions\
         of the steps are cited from DADA2 tutorial as well.\
        \n\nMultiple sequence alignment of resolved sequence variants is used to generate a phylogenetic tree,\
        which is required for calculating UniFrac beta-diversity distances between microbiome samples.\n\n\n"
    
    captions["dada2errorintro"]='\n\n "The DADA2 algorithm makes use of a parametric error model (err) and every amplicon dataset has a different set of error rates. \
        \n\nThe learnErrors method learns the error model from the data, by alternating estimation of the error rates and inference of \
        sample composition until they converge on a jointly consistent solution.\n\nAs in many machine-learning problems, the algorithm must \
        begin with an initial guess, for which the maximum possible error rates in this data are used \
        the error rates if only the most abundant sequence is correct and all the rest are errors." [DADA2 Tutorial]\n'
    
    captions["dada2errorinfo"]='"The error rates for each possible transition (eg. A->T,A->G, etc) are shown. \
        Points are the observed error rates for each consensus quality score. \
        \n\nThe black line shows the estimated error rates after convergence. \
         The red line shows the error rates expected under the nominal definition of the Q-value. \
        \n\nIf the black line (the estimated rates) fits the observed rates well, \
        and the error rates drop with increased quality as expected, then everything looks reasonable \
        and can proceed with confidence." [DADA2 Tutorial]\n\n'

    captions["dada2stepsinfo"]='\n\n "Dereplication combines all identical sequencing reads into into unique sequences with a corresponding abundance:\
        the number of reads with that unique sequence .... DADA2 retains a summary of the quality information associated with each unique sequence." [DADA2 Tutorial]\
        \n\nThe consensus quality profile of a unique sequence is the average of the positional qualities from the dereplicated reads.\
        \n\nThese quality profiles inform the error model of the subsequent denoising step, significantly increasing DADA2s accuracy. \
        \n\nThe sample inference step performs the core sequence-variant inference algorithm to the dereplicated data. \
        Spurious sequence variants are further reduced by merging overlapping reads. \n\nThe core function here is mergePairs \
        which depends on the forward and reverse re.samples being in matching order at the time they were dereplicated \
        \n\n "The core dada method removes substitution and indel errors, but chimeras remain.\
        Fortunately, the accuracy of the sequences after denoising makes identifying chimeras simpler than it is when dealing with fuzzy OTUs\
        all sequences which can be exactly reconstructed as a bimera (two-parent chimera) from more abundant sequence." [DADA2 Tutorial]' \
         + '\n\n "Most of reads should remain after chimera removal (it is not uncommon for a majority of sequence variants to be removed though).\
        If most of your reads were removed as chimeric, upstream processing may need to be revisited.\
        In almost all cases this is caused by primer sequences with ambiguous nucleotides that were not removed prior to beginning the DADA2 pipeline." [DADA2 Tutorial]\n'
    
    captions["dada2countsinfo"]='This figure shows the number of reads that made it through each step in the pipeline\
        \n\nThere should no be a step in which a majority of reads are lost, except filtering when it is stringent.\
        \n\n "If a majority of reads failed to merge, you may need to revisit the  truncLen parameter used in the filtering step\
        and make sure that the truncated reads span your amplicon." [DADA2 Tutorial]\
        \n\nIf a majority of reads failed to pass the chimera check, you may need to revisit the removal of primers,\
        as the ambiguous nucleotides in unremoved primers interfere with chimera identification.\n'

    captions["dada2taxinfo"]='"The assignTaxonomy function takes a set of sequences and a training set of taxonomically classified sequences,\
        and outputs the taxonomic assignments with at least minBoot bootstrap confidence." [DADA2 Tutorial]\
        \n\n assignTaxonomy(... ) implements the RDP naive Bayesian classifier method described in Wang et al. 2007.'  \
        + " In short, the kmer profile of the sequences to be classified are compared against the kmer profiles of all sequences in a training set\
        of sequences with assigned taxonomies. The reference sequence with the most similar profile is used to assign taxonomy to the query sequence,\
        and then a bootstrapping approach is used to assess the confidence assignment at each taxonomic level.\n \n"
 
          
    captions["usearchcountsinfo"]="This figure shows counts of reads in three categories: \n \
        \n1) classified: reads that align to OTUs with known taxonomy,\n2) unclassified: reads that align to OTUs of unknown taxonomy,\n3) unmapped: reads that do not align to any OTUs.\n\n The sum of these\
        three read counts for each sample is the total original read count not including filtering prior to OTU clustering.\n"
     




#+ echo=False

from anadama2 import PweaveDocument
from anadama2.reporters import LoggerReporter 

document=PweaveDocument()  

# get the variables for this document generation task
vars = document.get_vars()

# determine the document format
pdf_format = True if vars["format"] == "pdf" else False

#' # Data Processing Workflow Information

#' ## Software Versions

#' 
#' <% for version in LoggerReporter.read_log(vars["log"],"versions"): print("  * " + version+"  ") %>
#'

#' ## Tasks Run

#'
#' <% for command in LoggerReporter.read_log(vars["log"],"commands"): print("  * " + command+"  ") %>
#' 



#!/usr/bin/env python

"""
bioBakery Workflows: whole metagenome and metatranscriptome shotgun workflow for visualization

Copyright (c) 2017 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os

# import the workflow class from anadama2
from anadama2 import Workflow

# import the document templates from biobakery_workflows
from biobakery_workflows import utilities

# import the files for descriptions and paths
from biobakery_workflows import files

# create a workflow instance, providing the version number and description
# remove the input folder option as it will be replaced with multiple input files
workflow = Workflow(version="0.1", remove_options=["input"],
                    description="A workflow for whole metagenome and metatranscriptome shotgun sequence visualization")

# list the required and optional files for the workflow
# these are expected to be included in the input folder
wmgx_input_files={"required":["kneaddata_read_counts","taxonomic_profile","pathabundance_relab"],"optional":["humann2_read_counts","feature_counts"]}
wmtx_input_files={"required":["kneaddata_read_counts"],"optional":["humann2_read_counts","feature_counts"]}
norm_input_files={"optional":["genefamilies_norm_ratio","ecs_norm_ratio","paths_norm_ratio"]}

# create a custom description for the input argument listing all expected input files
input_desc="A folder containing the final products from the wmgx_mwtx data workflow.\n\nThe input folder should include the following:\n\n"
input_desc+="Whole Metagenome Shotgun\n---------------------------------\n"
input_desc+=files.ShotGun.list_file_path_description(files.ShotGun.wmgx_folder_name,wmgx_input_files)
input_desc+="\n\nWhole Metatranscriptome Shotgun\n---------------------------------\n"
input_desc+=files.ShotGun.list_file_path_description(files.ShotGun.wmtx_folder_name,wmtx_input_files)
input_desc+="\n\nRNA/DNA Norm\n---------------------------------\n"
input_desc+=files.ShotGun.list_file_path_description("",norm_input_files)

# add the custom arguments to the workflow                          
workflow.add_argument("input",desc=input_desc,required=True)
workflow.add_argument("project-name",desc="the name of the project", required=True)
workflow.add_argument("introduction-text",desc="the text to include in the intro of the report",
    default="The data was run through the standard workflow for whole metagenome and metatranscriptome shotgun sequencing.")
workflow.add_argument("exclude-workflow-info",desc="do not include data processing task info in report", action="store_true")
workflow.add_argument("format",desc="the format for the report", default="pdf", choices=["pdf","html"])

# get the arguments from the command line
args = workflow.parse_args()

# set the input folders for the wmgx and wmtx files
wmgx_input_folder=os.path.join(args.input,files.ShotGun.wmgx_folder_name)
wmtx_input_folder=os.path.join(args.input,files.ShotGun.wmtx_folder_name)

# get the paths for the required files and check they are found
wmgx_qc_counts=files.ShotGun.path("kneaddata_read_counts",wmgx_input_folder, error_if_not_found=True)
wmtx_qc_counts=files.ShotGun.path("kneaddata_read_counts",wmtx_input_folder, error_if_not_found=True)
taxonomic_profile=files.ShotGun.path("taxonomic_profile",wmgx_input_folder, error_if_not_found=True)
pathabundance=files.ShotGun.path("pathabundance_relab",wmgx_input_folder, error_if_not_found=True)

# get the templates for the report
templates=[utilities.get_package_file("header"),
    utilities.get_package_file("quality_control_paired_dna_rna"),
    utilities.get_package_file("taxonomy"),
    utilities.get_package_file("functional_dna_rna")]

# add the template for the data processing information
log_file=None
if not args.exclude_workflow_info:
    templates+=[utilities.get_package_file("workflow_info")]
    log_file=files.Workflow.path("log", args.input, error_if_not_found=True)

# add the document to the workflow
doc_task=workflow.add_document(
    templates=templates,
    depends=[wmgx_qc_counts, wmtx_qc_counts,
             taxonomic_profile, pathabundance], 
    targets=workflow.name_output_files("wmgx_wmtx_report."+args.format),
    vars={"title":"Metagenome and Metatranscriptome Report",
          "project":args.project_name,
          "introduction_text":args.introduction_text,
          "dna_read_counts":wmgx_qc_counts,
          "rna_read_counts":wmtx_qc_counts,
          "dna_aligned_read_counts":files.ShotGun.path("humann2_read_counts",wmgx_input_folder, none_if_not_found=True),
          "rna_aligned_read_counts":files.ShotGun.path("humann2_read_counts",wmtx_input_folder, none_if_not_found=True),
          "dna_feature_counts":files.ShotGun.path("feature_counts",wmgx_input_folder, none_if_not_found=True),
          "rna_feature_counts":files.ShotGun.path("feature_counts",wmtx_input_folder, none_if_not_found=True),
          "taxonomic_profile":taxonomic_profile,
          "dna_pathabundance":pathabundance,
          "genefamilies_norm_ratio":files.ShotGun.path("genefamilies_norm_ratio",args.input,none_if_not_found=True),
          "ecs_norm_ratio":files.ShotGun.path("ecs_norm_ratio",args.input,none_if_not_found=True),
          "paths_norm_ratio":files.ShotGun.path("paths_norm_ratio",args.input,none_if_not_found=True),
          "format":args.format,
          "log":log_file},
    table_of_contents=True)

# add an archive of the document and figures, removing the log file
# the archive will have the same name and location as the output folder
workflow.add_archive(
    depends=[args.output,doc_task],
    targets=args.output+".zip",
    remove_log=True)

# start the workflow
workflow.go()



"""
bioBakery Workflows: tasks.dadatwo module
A collection of tasks for DADA2 workflow with 16s amplicon sequences

Copyright (c) 2017 Harvard School of Public Health

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""
from anadama2.tracked import TrackedDirectory, TrackedExecutable
from biobakery_workflows import files, config, utilities
import os,fnmatch

def remove_primers(workflow,fwd_primer,rev_primer,input_folder,output_folder,pair_id,threads):
    """ Identifies primers and N filters samples
       Args:
           workflow (anadama2.workflow): an instance of the workflow class
           input_folder (string): path to input folder
           output_folder (string):  path to output folder
           fwd_primer (string): forward primer
           rev_primer (string): reverse primer
           pair_id (string): pair identifier
           threads (string): number of threads

       Requires:
          dada2, Biostrings, ShortRead, tools r packages

       Returns:
           string: path to folder with primers removed files
    """
    script_path = utilities.get_package_file("identify_primers", "Rscript")
    filtN_folder = os.path.join(output_folder,"filtN")
    primers_folder = os.path.join(output_folder,"primers")
    fwd_primer_file = os.path.join(primers_folder,"fwd_primer_file.txt")
    rev_primer_file = os.path.join(primers_folder,"rev_primer_file.txt")
    cutadapt_folder = os.path.join(output_folder, "cutadapt")

    # run identify primers task
    workflow.add_task(
        "[vars[0]]  \
          --input_dir=[args[3]] \
          --filtn_dir=[vars[1]] \
          --primers_dir=[vars[2]] \
          --threads=[args[4]] \
          --fwd_primer_file=[targets[0]] \
          --rev_primer_file=[targets[1]] \
          --fwd_primer=[args[0]] \
          --rev_primer=[args[1]] \
          --pair_id=[args[2]]",
        targets=[fwd_primer_file,rev_primer_file,
                 TrackedDirectory(filtN_folder)],
        args=[fwd_primer, rev_primer, pair_id,input_folder,threads],
        vars=[script_path,filtN_folder,primers_folder,output_folder],
        name="identify_primers"
    )

    pair_id2 = pair_id.replace("1", "2",1)
    fwd_files = sorted(fnmatch.filter(os.listdir(input_folder), "*"+pair_id+"*.fastq*"))
    rev_files = sorted(fnmatch.filter(os.listdir(input_folder), "*" + pair_id2 + "*.fastq*"))

    #run cutadapt to remove primers
    for i in range(0,len(fwd_files)):
        fwd_file=os.path.join(input_folder,fwd_files[i])
        rev_file = os.path.join(input_folder, rev_files[i])
        workflow.add_task(
            cutadapt_do,
            depends=[fwd_primer_file,
                     rev_primer_file,
                     fwd_file,
                     rev_file,
                     TrackedDirectory(filtN_folder),
                     TrackedExecutable("cutadapt",version_command="echo 'cutadapt' `cutadapt --version`")],
            targets=[TrackedDirectory(cutadapt_folder)],
            name="remove_primers"
        )

    return cutadapt_folder


def cutadapt_do(task):

    """Reads primers from the files and runs cutadapt task
           Args:
               task (anadama2.task): an instance of the task class"""

    from anadama2.util import get_name

    with open(get_name(task.depends[0])) as f:
        FWD = f.read().splitlines()
    with open(get_name(task.depends[1])) as f:
        REV = f.read().splitlines()

    cutadapt_folder=get_name(task.targets[0])
    filtN_folder = get_name(task.depends[4])
    fwd_filename=os.path.basename(get_name(task.depends[2]))
    rev_filename = os.path.basename(get_name(task.depends[3]))

    fwd_reads_out=os.path.join(cutadapt_folder,fwd_filename)
    rev_reads_out = os.path.join(cutadapt_folder,rev_filename)
    fwd_reads_in = os.path.join(filtN_folder,fwd_filename)
    rev_reads_in = os.path.join(filtN_folder,rev_filename)

    if not os.path.exists(cutadapt_folder):
        os.mkdir(cutadapt_folder)

    command="cutadapt -g "+FWD[0]+" -a "+REV[1]+" -G "+REV[0]+" -A "+FWD[1]+" -n 2 -o "+fwd_reads_out+\
                       " -p "+rev_reads_out+" "+fwd_reads_in+" "+ rev_reads_in+" --minimum-length 10"
    
    #run task
    utilities.run_task(command, depends=task.depends, targets=task.targets)


def filter_trim(workflow,input_folder,output_folder,maxee,trunc_len_max,pair_id,threads):
    
         """ Filters samples by maxee and trims them, renders quality control plots
         of forward and reverse reads for each sample, creates read counts tsv and rds files.
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                input_folder (string): path to input folder
                output_folder (string):  path to output folder
                maxee (string): maxee value to use for filtering
                trunc_len_max (string): max length for truncating reads
                pair_id (string): pair identifier
                threads (int): number of threads
                
            Requires:
               dada2, gridExtra,tools r packages
                
            Returns:
                string: path to file that contains read counts before and after filtering
                string: path to folder with filtered and trimmed sample files
         """
         reads_plotF_png = files.SixteenS.path("readF_qc", output_folder)
         reads_plotR_png = files.SixteenS.path("readR_qc", output_folder)

         readcounts_tsv_path = os.path.join(output_folder, "Read_counts_after_filtering.tsv")
         readcounts_rds_path = os.path.join(output_folder, "Read_counts_filt.rds")
         filtered_dir = "filtered_input"
         script_path = utilities.get_package_file("filter_and_trim", "Rscript")
         workflow.add_task(
             "[vars[0]] \
               --input_dir=[args[0]]\
               --output_dir=[args[1]]\
               --filtered_dir=[vars[1]]\
               --maxee=[args[2]]\
               --trunc_len_max=[args[3]]\
               --readcounts_tsv_path=[targets[0]]\
               --readcounts_rds_path=[targets[1]]\
               --reads_plotF=[targets[2]]\
               --reads_plotR=[targets[3]]\
               --pair_id=[args[4]]\
               --threads=[args[5]]",
             depends =[TrackedDirectory(input_folder)],
             targets = [readcounts_tsv_path, readcounts_rds_path, reads_plotF_png, reads_plotR_png],
             args = [input_folder, output_folder, maxee, trunc_len_max, pair_id, threads],
             vars = [script_path,filtered_dir],
             name ="filter_and_trim"
             )
         return readcounts_tsv_path, filtered_dir
     

def learn_error(workflow, output_folder, filtered_dir, readcounts_tsv_path, threads):
    
         """ Learns error rates for each sample, renders error rates plots for forward and reverse reads
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                output_folder (string): path to output folder
                filtered_dir (string): path to directory with filtered files
                readcounts_tsv_path (string): path to read counts after filtering tsv file
                threads (int): number of threads

            Requires:
                dada2, ggplot2 r packages

            Returns:
                string: path to file that contains error rates of forward reads
                string: path to file that contains error rates of reverse reads
         """

         error_ratesF_png = files.SixteenS.path("error_ratesF", output_folder)
         error_ratesR_png = files.SixteenS.path("error_ratesR", output_folder)
         
         error_ratesF_path= os.path.join(output_folder, "error_ratesFWD.rds")
         error_ratesR_path =os.path.join(output_folder, "error_ratesREV.rds")

         script_path = utilities.get_package_file("learn_error_rates", "Rscript")
       
         workflow.add_task(
             "[vars[0]] \
               --output_dir=[args[0]]\
               --filtered_dir=[args[1]]\
               --error_ratesF_png=[targets[0]]\
               --error_ratesR_png=[targets[1]]\
               --error_ratesF_path=[targets[2]]\
               --error_ratesR_path=[targets[3]]\
               --threads=[vars[1]]",
             depends = [readcounts_tsv_path],
             targets = [error_ratesF_png, error_ratesR_png, error_ratesF_path, error_ratesR_path],  
             args = [output_folder, filtered_dir],
             vars = [script_path, threads],
             name = "learn_error_rates"
             )
         return error_ratesF_path, error_ratesR_path
     

def merge_paired_ends(workflow, output_dir, filtered_dir, error_ratesF_path, error_ratesR_path, threads, minoverlap, maxmismatch):
    
        """ Dereplicates and merges paired reads
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                output_folder (string): path to output folder
                filtered_dir (string): path to directory with filtered files
                error_ratesF_path (string): path to rds file that contains error rates of forward reads
                error_ratesR_path (string): path to rds file that contains error rates of reverse reads
                threads (int): number of threads
                minoverlap (int): the min number of pairs for overlap for the merge step
                maxmismatch (int): the max number of mismatch for pairs to merge
            Requires:
                dada2, tools r packages
                
            Returns:
                string: path to rds file that contains merged and dereplicated reads
         """

        mergers_file_path = os.path.join(output_dir, "mergers.rds")
        script_path = utilities.get_package_file("merge_paired_ends", "Rscript")
        
        workflow.add_task(
            "[vars[0]] \
              --output_dir=[args[0]]\
              --filtered_dir=[args[1]]\
              --error_ratesF_path=[depends[0]]\
              --error_ratesR_path=[depends[1]]\
              --mergers_file_path=[targets[0]]\
              --threads=[vars[1]]\
              --minoverlap=[args[2]]\
              --maxmismatch=[args[3]]",
            depends = [error_ratesF_path, error_ratesR_path],
            targets = [mergers_file_path],                       
            args = [output_dir, filtered_dir, minoverlap, maxmismatch],
            vars = [script_path, threads],
            name = "dereplicate_and_merge"
            )
        return mergers_file_path
    

def const_seq_table(workflow, output_folder, filtered_dir,  mergers_file_path, threads):
    
         """ Builds ASV table, removes chimeras, creates read counts at each step, and fasta file with all sequences
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                output_folder (string):  path to output folder
                filtered_dir (string): path to directory with filtered files
                mergers_file_path (string): path to rds file that contains merged reads
                threads (int): number of threads
                
            Requires:
                dada2, tools, seqinr r packages

            Returns:
                string: path to rds file that contains ASV data
                string: path to read counts at each step tsv file
                string: path to fasta file with all sequences
         """
         
         read_counts_steps_path = files.SixteenS.path("counts_each_step", output_folder)
         
         seqtab_file_path = os.path.join(output_folder, "seqtab_final.rds")
         seqs_fasta_path = os.path.join(output_folder, "sequences.fasta")
         readcounts_rds = "Read_counts_filt.rds"
         asv_tsv = "all_samples_SV_counts.tsv"

         script_path = utilities.get_package_file("const_seq_table", "Rscript")
         version_script = utilities.get_package_file("dada2_version", "Rscript")

         version_command = """echo 'r' `r -e 'packageVersion("dada2")' | grep -C 1 dada2`"""

         workflow.add_task(
            "[vars[0]] \
              --output_dir=[args[0]]\
              --filtered_dir=[args[1]]\
              --merged_file_path=[depends[0]]\
              --read_counts_steps_path=[targets[0]]\
              --readcounts_rds=[vars[2]]\
              --asv_tsv=[vars[3]]\
              --seqtab_file_path=[targets[1]]\
              --seqs_fasta_path=[targets[2]]\
              --threads=[vars[1]]",
            depends = [mergers_file_path,TrackedExecutable("R", version_command="echo '" +  version_script + "' `" + version_script + "`")],
            targets = [read_counts_steps_path, seqtab_file_path, seqs_fasta_path],
            args = [output_folder, filtered_dir],
            vars = [script_path, threads, readcounts_rds, asv_tsv ],
            name = "construct_sequence_table"
            )
         return seqtab_file_path, read_counts_steps_path, seqs_fasta_path


def assign_taxonomy(workflow, output_folder, seqtab_file_path, ref_path, threads):
    
         """ Assigns taxonomy using green genes, silva, or rdp database, creates closed reference file
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                output_folder (string): path to output folder
                seqtab_file_path (string): path to rds file that contains ASV data
                ref_path (string): reference database name
                threads (int):
                
            Requires:
                dada2 r package
                
            Returns:
                string: path to closed reference file
         """

         otu_closed_ref_path  = files.SixteenS.path("otu_table_closed_reference", output_folder)

         # check what reference db to use for taxonomy assignment
         if ref_path == "unite":
             refdb_path = config.SixteenS().unite
             refdb_species_path = "None"
         elif ref_path == "silva":
             refdb_path = config.SixteenS().silva_dada2 
             refdb_species_path = config.SixteenS().silva_species_dada2 
         elif ref_path == "rdp":
             refdb_path = config.SixteenS().rdp_dada2 
             refdb_species_path = config.SixteenS().rdp_species_dada2  
         else:    
             refdb_path = config.SixteenS().greengenes_dada2
             refdb_species_path = "None"

         script_path = utilities.get_package_file("assign_taxonomy", "Rscript")
             
         workflow.add_task(
            "[vars[2]] \
              --output_dir=[args[0]]\
              --refdb_path=[vars[0]]\
              --refdb_species_path=[vars[1]]\
              --seqtab_file_path=[depends[0]]\
              --otu_closed_ref_path=[targets[0]]\
              --threads=[vars[3]]",
            depends = [seqtab_file_path],
            targets = [otu_closed_ref_path],                              
            args = [output_folder],
            vars =[refdb_path, refdb_species_path, script_path, threads],
            name = "assign_taxonomy"
            )         
     
         return otu_closed_ref_path
 
 
def remove_tmp_files(workflow, output_folder, otu_closed_ref_path, msa_fasta_path, fasttree_file_path):
    
         """ Removes temporary rds files
            
            Args:
                workflow (anadama2.workflow): an instance of the workflow class
                output_folder (string): path to output folder.
                otu_closed_ref_path (string): path to closed reference file
                msa_fasta_path (string): path to msa file
                fasttree_file_path (string): path to phylogenetic tree file

            Requires:
               None
                
            Returns:
               None
          """
         
         rm_out_file = os.path.join(output_folder, "tmp_rm.txt")
             
         workflow.add_task(
                     "rm  [args[0]]/*.rds  &>[targets[0]] ",
                     depends = [otu_closed_ref_path, msa_fasta_path, fasttree_file_path],
                     args = [output_folder],
                     targets = [rm_out_file],
                     name = "rm_tmp_files"
                     )
         



#!/usr/bin/env python

"""
This script will create fasta files for each genus/species taxonomy from the alignments 
These fasta output files will then be used as input for the oligotyping workflow.

To run:
$ python create_fasta_per_taxonomy_from_alignments.py all_samples_taxonomy_closed_reference.tsv all_samples_otu_mapping_results.uc  all_samples_green_genes_mapping_results.uc all_samples_truncated.fasta output_folder

"""

import sys

try:
    import argparse
except ImportError:
    sys.exit("Please upgrade to at least python v2.7")

import os
import re
from operator import add

USEARCH_HIT="H"
USEARCH_HIT_INDEX=0
USEARCH_QUERY_INDEX=8
USEARCH_TARGET_INDEX=9
SAMPLE_READ_DELIMITER="."
OLIGOTYPING_READ_DELIMITER="_"
UNNAMED_TAXONOMY="Unclassified"
FASTA_SEQ_START=">"

TAXONOMY_IDS={"order":"o__","family":"f__","genus":"g__","species":"s__"}

def catch_open(file,write=None):
    """ Try to open the file, return file handle, or exit if unable to open """
    try:
        if write:
            file_handle=open(file,"w")
        else:
            file_handle=open(file)
    except EnvironmentError:
        sys.exit("ERROR: Unable to open file: " + file)
        
    return file_handle

def get_sample_id(sample):
    """ Get the sample id from a string with the read number """
    return SAMPLE_READ_DELIMITER.join(sample.split(SAMPLE_READ_DELIMITER)[0:-1])

def create_fasta_files(matches, all_taxa, trimmed_fasta, output):
    """ Create fasta files organized by taxonomy """

    # write out all of the sequences for a single taxonomy at a time   
    for taxa in all_taxa:
        # try to open the output file
        current_output=os.path.join(output,taxa+".fasta")
        print("Writing file: " + current_output)
        file_handle_write=catch_open(current_output,write=True)
        write_seq=False
        with catch_open(trimmed_fasta) as file_handle_read:
            for line in file_handle_read:
                if line[0] == FASTA_SEQ_START:
                    id=line.rstrip().replace(FASTA_SEQ_START,"")
                    id_taxonomy=matches.get(id,"")
                    if id_taxonomy == taxa:
                        file_handle_write.write(line.replace(SAMPLE_READ_DELIMITER, OLIGOTYPING_READ_DELIMITER))
                        write_seq=True
                    else:
                        write_seq=False
                else:
                    if write_seq:
                        file_handle_write.write(line)
        file_handle_write.close()

def process_alignments(input_otu, input_nonchimera_uc, input_greengenes_uc,rank):
    """ Read in the alignment information to get taxonomy for each read """

    # read in the taxonomy from the otu table
    taxonomy={}
    header=""
    print("Reading otu table")
    for data in read_tab_file(input_otu):
        if not header:
            header=data[1:]
        else:
            # process the taxonomy for each otu 
            taxonomy_data=data[-1].replace(" ","").split(";")
            # filter to the rank provided 
            max_taxonomy=list(filter(lambda x: x.startswith(TAXONOMY_IDS[rank]), taxonomy_data))[0]
            max_taxonomy_index=taxonomy_data.index(max_taxonomy)
            # filter out those missing taxonomy information (ie filter out those that are "g__"
            # as these are "unclassified")
            if not (TAXONOMY_IDS[rank] == max_taxonomy):
                taxonomy[data[0]]="_".join(taxonomy_data[:(max_taxonomy_index+1)]).replace("[","").replace("]","")

    # read the green genes alignment
    otu_to_ggid={}
    print("Reading greengenes uc file")
    for otu_id, target in read_uc_file(input_greengenes_uc):
        otu_to_ggid[otu_id]=target

    # read the cluster alignment
    read_to_otuid={}
    print("Reading otu uc file")
    for read_id, otu_id in read_uc_file(input_nonchimera_uc):
        read_to_otuid[read_id]=otu_id

    # create a data set of read id to taxonomy
    matches={}
    all_taxa=set()
    print("Matching read ids to taxonomy")
    for readid,otuid in read_to_otuid.items():
        # get the greengenes id and taxonomy
        ggid=otu_to_ggid.get(otuid,"")
        taxon=taxonomy.get(ggid,"")

        if taxon:
            matches[readid]=taxon
            all_taxa.add(taxon)

    return matches, all_taxa

def read_tab_file(file):
    """ Read a tab delimited file """
    with catch_open(file) as file_handle:
        for line in file_handle:
            yield line.rstrip().split("\t")
    
def read_uc_file(uc_file, all_alignments=None):
    """ Read the uc file and return the type, query, and target """
    for data in read_tab_file(uc_file):
        try:
            query = data[USEARCH_QUERY_INDEX]
        except IndexError:
            query = ""
                  
        try:
            target = data[USEARCH_TARGET_INDEX]
        except IndexError:
            target = ""
                
        try:
            type = data[USEARCH_HIT_INDEX]
        except IndexError:
            type = ""
                
        if type and query and target:
            if all_alignments:
                yield type, query, target
            elif type == USEARCH_HIT:
                yield query, target
    
def parse_arguments(args):
    """ Parse the arguments from the user"""
    
    parser=argparse.ArgumentParser(description="Process usearch output to get taxonomy.")
    parser.add_argument('input_otu',help="The closed reference otu table.")
    parser.add_argument('input_nonchimera_uc',help="The uc file of alignment results from usearch to nonchimeras.")
    parser.add_argument('input_greengenes_uc',help="The uc file of alignment results from nonchimeras to greengenes.")
    parser.add_argument('input_trimmed_fasta',help="The fasta file of trimmed reads from the samples.")
    parser.add_argument('--rank',help="The rank to user for the output fasta files.",choices=TAXONOMY_IDS.keys(),default="species")
    parser.add_argument('output_folder',help="The folder to write the per genus/species output files.")
    
    return parser.parse_args()

def check_file_nonempty(file):
    """ Check the file is non-empty """
    
    try:
        size = os.stat(file).st_size
    except EnvironmentError:
        sys.exit("ERROR: Unable to access input file: " + file)
        
    if size == 0:
        sys.exit("ERROR: Input file is empty: " + file)
        
def main():
    # parse arguments
    args = parse_arguments(sys.argv)
    
    # check the input files are non-empty
    check_file_nonempty(args.input_nonchimera_uc)

    # if the output folder does not exist, then create it
    if not os.path.isdir(args.output_folder):
        try:
            print("Creating output directory: " + args.output_folder)
            os.mkdir(args.output_folder)
        except EnvironmentError:
            sys.exit("ERROR: Unable to create output directory")

    matches, all_taxa=process_alignments(args.input_otu,args.input_nonchimera_uc,args.input_greengenes_uc,args.rank)
    create_fasta_files(matches,all_taxa,args.input_trimmed_fasta,args.output_folder)

if __name__ == "__main__":
    main()

    
    



#!/usr/bin/env python

import sys
import os
import argparse

# This script will count all features for each sample. It can ignore stratification by species
# and also "un"-features if these options are set by the user.

def parse_arguments(args):
    """ 
    Parse the arguments from the user
    """
    parser = argparse.ArgumentParser(
        description= "Count features for each species\n",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "-i", "--input",
        help="the feature table\n[REQUIRED]",
        metavar="<input.tsv>",
        required=True)
    parser.add_argument(
        "-o", "--output",
        help="file to write counts table\n[REQUIRED]",
        metavar="<output>",
        required=True)
    parser.add_argument(
        "--reduce-sample-name",
        help="remove the extra strings from the sample names",
        action='store_true')
    parser.add_argument(
        "--ignore-un-features",
        help="do not count UNMAPPED, UNGROUPED, or UNINTEGRATED features",
        action='store_true')
    parser.add_argument(
        "--ignore-stratification",
        help="only count the main feature not the stratified instances",
        action='store_true')
    parser.add_argument(
        "--include",
        help="only count features with this string included")
    parser.add_argument(
        "--filter",
        help="do not count features with this string included")

    return parser.parse_args()


def main():

    args=parse_arguments(sys)

    data=[]
    samples=[]
    with open(args.input) as file_handle:
        # remove RPKs from sample name if included
        if args.reduce_sample_name:
            samples=[sample.replace("_Abundance-RPKs","").replace("_genefamilies_Abundance","").replace("_Abundance","").replace("_taxonomic_profile","") for sample in file_handle.readline().rstrip().split("\t")[1:]]
        else:
            samples=file_handle.readline().rstrip().split("\t")[1:]
             
        for line in file_handle:
            if "|" in line and args.ignore_stratification:
                store=False
            else:
                store=True

            if "UNMAPPED" in line or "UNGROUPED" in line or "UNINTEGRATED" in line and args.ignore_un_features:
                store=False

            if args.include and not args.include in line:
                store=False

            if args.filter and args.filter in line:
                store=False

            if store:
                data.append(line.rstrip().split("\t")[1:])

    try:
        file_handle=open(args.output,"w")
    except EnvironmentError:
        sys.exit("Error: Unable to open output file: " + args.output)

    # write out the header
    file_handle.write("\t".join(["# samples","total features"])+"\n")

    # count the total features for each sample
    for i, sample in enumerate(samples):
        features=0
        for row in data:
            if float(row[i]) > 0:
                features+=1
        file_handle.write(sample+"\t"+str(features)+"\n")

    file_handle.close()

if __name__ == "__main__":
    main()





#!/usr/bin/env python

# This script will print out a bar codes file with reverse complement sequences.

import sys
import os
import string
import argparse

def parse_arguments(args):
    """ 
    Parse the arguments from the user
    """
    
    parser = argparse.ArgumentParser(
        description= "Reverse complement barcodes (add another group of barcodes to the file)\n",
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "-i","--input",
        help="the barcodes tsv file (format: <id>\t<barcode>)\n",
        required=True)
    parser.add_argument(
        "-o","--output",
        help="the new barcodes tsv file (format: <id>\t<barcode>\t<group>)\n",
        required=True)

    return parser.parse_args()

def main():
    # parse the arguments
    args=parse_arguments(sys.argv)
    
    input=os.path.abspath(args.input)
    output=os.path.abspath(args.output)
    
    if not os.path.isfile(input):
        sys.exit("The input file provided can not be found: " + 
            input+"  Please provide the full path or enter a new file.")
    # try to open the input file
    try:
        file_handle_read=open(input,"rt")
    except EnvironmentError:
        sys.exit("Unable to open input file.")

    # try to open the output file
    try:
        file_handle_write=open(output,"wt")
    except EnvironmentError:
        sys.exit("Unable to open output file.")

    rev_cmp=string.maketrans("ACGT","TGCA")

    newlines=[]
    # write the header
    header=file_handle_read.readline()
    file_handle_write.write(header)
    for line in file_handle_read:
        data=line.rstrip().split("\t")
        file_handle_write.write("\t".join([data[0],data[1],"original"])+"\n")
        # reverse complement the bar code
        newlines.append("\t".join([data[0],data[1].translate(rev_cmp)[::-1],"reverse_complement"]))
        
    # print the reverse complements
    for line in newlines:
        file_handle_write.write(line+"\n")

    file_handle_write.close()
    file_handle_read.close()

if __name__ == "__main__":
    main()

